# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..fix import Cleaner


def test_Cleaner_inputs():
    input_map = dict(
        aggressive=dict(
            argstr='-A',
            position=3,
        ),
        args=dict(argstr='%s', ),
        artifacts_list_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        cleanup_motion=dict(
            argstr='-m',
            position=2,
        ),
        confound_file=dict(
            argstr='-x %s',
            extensions=None,
            position=4,
        ),
        confound_file_1=dict(
            argstr='-x %s',
            extensions=None,
            position=5,
        ),
        confound_file_2=dict(
            argstr='-x %s',
            extensions=None,
            position=6,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        highpass=dict(
            argstr='-m -h %f',
            position=2,
            usedefault=True,
        ),
    )
    inputs = Cleaner.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Cleaner_outputs():
    output_map = dict(cleaned_functional_file=dict(extensions=None, ), )
    outputs = Cleaner.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import Allineate


def test_Allineate_inputs():
    input_map = dict(
        allcostx=dict(
            argstr='-allcostx |& tee %s',
            extensions=None,
            position=-1,
            xor=[
                'out_file', 'out_matrix', 'out_param_file', 'out_weight_file'
            ],
        ),
        args=dict(argstr='%s', ),
        autobox=dict(argstr='-autobox', ),
        automask=dict(argstr='-automask+%d', ),
        autoweight=dict(argstr='-autoweight%s', ),
        center_of_mass=dict(argstr='-cmass%s', ),
        check=dict(argstr='-check %s', ),
        convergence=dict(argstr='-conv %f', ),
        cost=dict(argstr='-cost %s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        epi=dict(argstr='-EPI', ),
        final_interpolation=dict(argstr='-final %s', ),
        fine_blur=dict(argstr='-fineblur %f', ),
        in_file=dict(
            argstr='-source %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        in_matrix=dict(
            argstr='-1Dmatrix_apply %s',
            extensions=None,
            position=-3,
            xor=['out_matrix'],
        ),
        in_param_file=dict(
            argstr='-1Dparam_apply %s',
            extensions=None,
            xor=['out_param_file'],
        ),
        interpolation=dict(argstr='-interp %s', ),
        master=dict(
            argstr='-master %s',
            extensions=None,
        ),
        maxrot=dict(argstr='-maxrot %f', ),
        maxscl=dict(argstr='-maxscl %f', ),
        maxshf=dict(argstr='-maxshf %f', ),
        maxshr=dict(argstr='-maxshr %f', ),
        newgrid=dict(argstr='-newgrid %f', ),
        nmatch=dict(argstr='-nmatch %d', ),
        no_pad=dict(argstr='-nopad', ),
        nomask=dict(argstr='-nomask', ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        nwarp=dict(argstr='-nwarp %s', ),
        nwarp_fixdep=dict(argstr='-nwarp_fixdep%s...', ),
        nwarp_fixmot=dict(argstr='-nwarp_fixmot%s...', ),
        one_pass=dict(argstr='-onepass', ),
        out_file=dict(
            argstr='-prefix %s',
            extensions=None,
            hash_files=False,
            name_source='in_file',
            name_template='%s_allineate',
            xor=['allcostx'],
        ),
        out_matrix=dict(
            argstr='-1Dmatrix_save %s',
            extensions=None,
            xor=['in_matrix', 'allcostx'],
        ),
        out_param_file=dict(
            argstr='-1Dparam_save %s',
            extensions=None,
            xor=['in_param_file', 'allcostx'],
        ),
        out_weight_file=dict(
            argstr='-wtprefix %s',
            extensions=None,
            xor=['allcostx'],
        ),
        outputtype=dict(),
        overwrite=dict(argstr='-overwrite', ),
        quiet=dict(argstr='-quiet', ),
        reference=dict(
            argstr='-base %s',
            extensions=None,
        ),
        replacebase=dict(argstr='-replacebase', ),
        replacemeth=dict(argstr='-replacemeth %s', ),
        source_automask=dict(argstr='-source_automask+%d', ),
        source_mask=dict(
            argstr='-source_mask %s',
            extensions=None,
        ),
        two_best=dict(argstr='-twobest %d', ),
        two_blur=dict(argstr='-twoblur %f', ),
        two_first=dict(argstr='-twofirst', ),
        two_pass=dict(argstr='-twopass', ),
        usetemp=dict(argstr='-usetemp', ),
        verbose=dict(argstr='-verb', ),
        warp_type=dict(argstr='-warp %s', ),
        warpfreeze=dict(argstr='-warpfreeze', ),
        weight=dict(argstr='-weight %s', ),
        weight_file=dict(
            argstr='-weight %s',
            deprecated='1.0.0',
            extensions=None,
            new_name='weight',
        ),
        zclip=dict(argstr='-zclip', ),
    )
    inputs = Allineate.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Allineate_outputs():
    output_map = dict(
        allcostx=dict(extensions=None, ),
        out_file=dict(extensions=None, ),
        out_matrix=dict(extensions=None, ),
        out_param_file=dict(extensions=None, ),
        out_weight_file=dict(extensions=None, ),
    )
    outputs = Allineate.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Nipype interfaces core
......................


Defines the ``Interface`` API and the body of the
most basic interfaces.
The I/O specifications corresponding to these base
interfaces are found in the ``specs`` module.

"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

from builtins import object, open, str, bytes

from copy import deepcopy
from datetime import datetime as dt
import os
import platform
import subprocess as sp
import shlex
import sys
import simplejson as json
from dateutil.parser import parse as parseutc
from future import standard_library
from traits.trait_errors import TraitError

from ... import config, logging, LooseVersion
from ...utils.provenance import write_provenance
from ...utils.misc import str2bool, rgetcwd
from ...utils.filemanip import (FileNotFoundError, split_filename,
                                which, get_dependencies)
from ...utils.subprocess import run_command

from ...external.due import due

from .traits_extension import traits, isdefined
from .specs import (BaseInterfaceInputSpec, CommandLineInputSpec,
                    StdOutCommandLineInputSpec, MpiCommandLineInputSpec,
                    get_filecopy_info)
from .support import (Bunch, InterfaceResult, NipypeInterfaceError,
                      format_help)

standard_library.install_aliases()

iflogger = logging.getLogger('nipype.interface')

PY35 = sys.version_info >= (3, 5)
PY3 = sys.version_info[0] > 2
VALID_TERMINAL_OUTPUT = [
    'stream', 'allatonce', 'file', 'file_split', 'file_stdout', 'file_stderr',
    'none'
]
__docformat__ = 'restructuredtext'


class Interface(object):
    """This is an abstract definition for Interface objects.

    It provides no functionality.  It defines the necessary attributes
    and methods all Interface objects should have.

    """

    input_spec = None  # A traited input specification
    output_spec = None  # A traited output specification
    _can_resume = False  # See property below
    _always_run = False  # See property below

    @property
    def can_resume(self):
        """Defines if the interface can reuse partial results after interruption.
        Only applies to interfaces being run within a workflow context."""
        return self._can_resume

    @property
    def always_run(self):
        """Should the interface be always run even if the inputs were not changed?
        Only applies to interfaces being run within a workflow context."""
        return self._always_run

    @property
    def version(self):
        """interfaces should implement a version property"""
        raise NotImplementedError

    @classmethod
    def _outputs(cls):
        """ Initializes outputs"""
        raise NotImplementedError

    @classmethod
    def help(cls, returnhelp=False):
        """ Prints class help """
        allhelp = format_help(cls)
        if returnhelp:
            return allhelp
        print(allhelp)
        return None  # R1710

    def __init__(self):
        """Subclasses must implement __init__"""
        raise NotImplementedError

    def run(self):
        """Execute the command."""
        raise NotImplementedError

    def aggregate_outputs(self, runtime=None, needed_outputs=None):
        """Called to populate outputs"""
        raise NotImplementedError

    def _list_outputs(self):
        """ List expected outputs"""
        raise NotImplementedError

    @classmethod
    def _get_filecopy_info(cls):
        """Provides information about file inputs to copy or link to cwd.
        Necessary for pipeline operation
        """
        iflogger.warning(
            '_get_filecopy_info member of Interface was deprecated '
            'in nipype-1.1.6 and will be removed in 1.2.0')
        return get_filecopy_info(cls)


class BaseInterface(Interface):
    """Implements common interface functionality.

    Implements
    ----------

    * Initializes inputs/outputs from input_spec/output_spec
    * Provides help based on input_spec and output_spec
    * Checks for mandatory inputs before running an interface
    * Runs an interface and returns results
    * Determines which inputs should be copied or linked to cwd

    This class does not implement aggregate_outputs, input_spec or
    output_spec. These should be defined by derived classes.

    This class cannot be instantiated.


    Relevant Interface attributes
    -----------------------------

    ``input_spec`` points to the traited class for the inputs
    ``output_spec`` points to the traited class for the outputs
    ``_redirect_x`` should be set to ``True`` when the interface requires
      connecting to a ``$DISPLAY`` (default is ``False``).
    ``resource_monitor`` if ``False`` prevents resource-monitoring this
      interface, if ``True`` monitoring will be enabled IFF the general
      Nipype config is set on (``resource_monitor = true``).


    """
    input_spec = BaseInterfaceInputSpec
    _version = None
    _additional_metadata = []
    _redirect_x = False
    references_ = []
    resource_monitor = True  # Enabled for this interface IFF enabled in the config
    _etelemetry_version_data = None

    def __init__(self, from_file=None, resource_monitor=None,
                 ignore_exception=False, **inputs):
        if config.getboolean('execution', 'check_version'):
            from ... import check_latest_version
            if BaseInterface._etelemetry_version_data is None:
                BaseInterface._etelemetry_version_data = check_latest_version()

        if not self.input_spec:
            raise Exception(
                'No input_spec in class: %s' % self.__class__.__name__)

        self.inputs = self.input_spec(**inputs)
        self.ignore_exception = ignore_exception

        if resource_monitor is not None:
            self.resource_monitor = resource_monitor

        if from_file is not None:
            self.load_inputs_from_json(from_file, overwrite=True)

            for name, value in list(inputs.items()):
                setattr(self.inputs, name, value)

    def _outputs(self):
        """ Returns a bunch containing output fields for the class
        """
        outputs = None
        if self.output_spec:
            outputs = self.output_spec()

        return outputs

    def _check_requires(self, spec, name, value):
        """ check if required inputs are satisfied
        """
        if spec.requires:
            values = [
                not isdefined(getattr(self.inputs, field))
                for field in spec.requires
            ]
            if any(values) and isdefined(value):
                if len(values) > 1:
                    fmt = ("%s requires values for inputs %s because '%s' is set. "
                           "For a list of required inputs, see %s.help()")
                else:
                    fmt = ("%s requires a value for input %s because '%s' is set. "
                           "For a list of required inputs, see %s.help()")
                msg = fmt % (self.__class__.__name__,
                             ', '.join("'%s'" % req for req in spec.requires),
                             name,
                             self.__class__.__name__)
                raise ValueError(msg)

    def _check_xor(self, spec, name, value):
        """ check if mutually exclusive inputs are satisfied
        """
        if spec.xor:
            values = [
                isdefined(getattr(self.inputs, field)) for field in spec.xor
            ]
            if not any(values) and not isdefined(value):
                msg = ("%s requires a value for one of the inputs '%s'. "
                       "For a list of required inputs, see %s.help()" %
                       (self.__class__.__name__, ', '.join(spec.xor),
                        self.__class__.__name__))
                raise ValueError(msg)

    def _check_mandatory_inputs(self):
        """ Raises an exception if a mandatory input is Undefined
        """
        for name, spec in list(self.inputs.traits(mandatory=True).items()):
            value = getattr(self.inputs, name)
            self._check_xor(spec, name, value)
            if not isdefined(value) and spec.xor is None:
                msg = ("%s requires a value for input '%s'. "
                       "For a list of required inputs, see %s.help()" %
                       (self.__class__.__name__, name,
                        self.__class__.__name__))
                raise ValueError(msg)
            if isdefined(value):
                self._check_requires(spec, name, value)
        for name, spec in list(
                self.inputs.traits(mandatory=None, transient=None).items()):
            self._check_requires(spec, name, getattr(self.inputs, name))

    def _check_version_requirements(self, trait_object, raise_exception=True):
        """ Raises an exception on version mismatch
        """
        unavailable_traits = []
        # check minimum version
        check = dict(min_ver=lambda t: t is not None)
        names = trait_object.trait_names(**check)

        if names and self.version:
            version = LooseVersion(str(self.version))
            for name in names:
                min_ver = LooseVersion(
                    str(trait_object.traits()[name].min_ver))
                if min_ver > version:
                    unavailable_traits.append(name)
                    if not isdefined(getattr(trait_object, name)):
                        continue
                    if raise_exception:
                        raise Exception(
                            'Trait %s (%s) (version %s < required %s)' %
                            (name, self.__class__.__name__, version, min_ver))

        # check maximum version
        check = dict(max_ver=lambda t: t is not None)
        names = trait_object.trait_names(**check)
        if names and self.version:
            version = LooseVersion(str(self.version))
            for name in names:
                max_ver = LooseVersion(
                    str(trait_object.traits()[name].max_ver))
                if max_ver < version:
                    unavailable_traits.append(name)
                    if not isdefined(getattr(trait_object, name)):
                        continue
                    if raise_exception:
                        raise Exception(
                            'Trait %s (%s) (version %s > required %s)' %
                            (name, self.__class__.__name__, version, max_ver))
        return unavailable_traits

    def _run_interface(self, runtime):
        """ Core function that executes interface
        """
        raise NotImplementedError

    def _duecredit_cite(self):
        """ Add the interface references to the duecredit citations
        """
        for r in self.references_:
            r['path'] = self.__module__
            due.cite(**r)

    def run(self, cwd=None, ignore_exception=None, **inputs):
        """Execute this interface.

        This interface will not raise an exception if runtime.returncode is
        non-zero.

        Parameters
        ----------

        cwd : specify a folder where the interface should be run
        inputs : allows the interface settings to be updated

        Returns
        -------
        results :  an InterfaceResult object containing a copy of the instance
        that was executed, provenance information and, if successful, results
        """
        from ...utils.profiler import ResourceMonitor

        # if ignore_exception is not provided, taking self.ignore_exception
        if ignore_exception is None:
            ignore_exception = self.ignore_exception

        # Tear-up: get current and prev directories
        syscwd = rgetcwd(error=False)  # Recover when wd does not exist
        if cwd is None:
            cwd = syscwd

        os.chdir(cwd)  # Change to the interface wd

        enable_rm = config.resource_monitor and self.resource_monitor
        self.inputs.trait_set(**inputs)
        self._check_mandatory_inputs()
        self._check_version_requirements(self.inputs)
        interface = self.__class__
        self._duecredit_cite()

        # initialize provenance tracking
        store_provenance = str2bool(
            config.get('execution', 'write_provenance', 'false'))
        env = deepcopy(dict(os.environ))
        if self._redirect_x:
            env['DISPLAY'] = config.get_display()

        runtime = Bunch(
            cwd=cwd,
            prevcwd=syscwd,
            returncode=None,
            duration=None,
            environ=env,
            startTime=dt.isoformat(dt.utcnow()),
            endTime=None,
            platform=platform.platform(),
            hostname=platform.node(),
            version=self.version)
        runtime_attrs = set(runtime.dictcopy())

        mon_sp = None
        if enable_rm:
            mon_freq = float(
                config.get('execution', 'resource_monitor_frequency', 1))
            proc_pid = os.getpid()
            iflogger.debug(
                'Creating a ResourceMonitor on a %s interface, PID=%d.',
                self.__class__.__name__, proc_pid)
            mon_sp = ResourceMonitor(proc_pid, freq=mon_freq)
            mon_sp.start()

        # Grab inputs now, as they should not change during execution
        inputs = self.inputs.get_traitsfree()
        outputs = None

        try:
            runtime = self._pre_run_hook(runtime)
            runtime = self._run_interface(runtime)
            runtime = self._post_run_hook(runtime)
            outputs = self.aggregate_outputs(runtime)
        except Exception as e:
            import traceback
            # Retrieve the maximum info fast
            runtime.traceback = traceback.format_exc()
            # Gather up the exception arguments and append nipype info.
            exc_args = e.args if getattr(e, 'args') else tuple()
            exc_args += (
                'An exception of type %s occurred while running interface %s.'
                % (type(e).__name__, self.__class__.__name__), )
            if config.get('logging', 'interface_level',
                          'info').lower() == 'debug':
                exc_args += ('Inputs: %s' % str(self.inputs), )

            runtime.traceback_args = ('\n'.join(
                ['%s' % arg for arg in exc_args]), )

            if not ignore_exception:
                raise
        finally:
            if runtime is None or runtime_attrs - set(runtime.dictcopy()):
                raise RuntimeError("{} interface failed to return valid "
                                   "runtime object".format(
                                       interface.__class__.__name__))
            # This needs to be done always
            runtime.endTime = dt.isoformat(dt.utcnow())
            timediff = parseutc(runtime.endTime) - parseutc(runtime.startTime)
            runtime.duration = (timediff.days * 86400 + timediff.seconds +
                                timediff.microseconds / 1e6)
            results = InterfaceResult(
                interface,
                runtime,
                inputs=inputs,
                outputs=outputs,
                provenance=None)

            # Add provenance (if required)
            if store_provenance:
                # Provenance will only throw a warning if something went wrong
                results.provenance = write_provenance(results)

            # Make sure runtime profiler is shut down
            if enable_rm:
                import numpy as np
                mon_sp.stop()

                runtime.mem_peak_gb = None
                runtime.cpu_percent = None

                # Read .prof file in and set runtime values
                vals = np.loadtxt(mon_sp.fname, delimiter=',')
                if vals.size:
                    vals = np.atleast_2d(vals)
                    runtime.mem_peak_gb = vals[:, 2].max() / 1024
                    runtime.cpu_percent = vals[:, 1].max()

                    runtime.prof_dict = {
                        'time': vals[:, 0].tolist(),
                        'cpus': vals[:, 1].tolist(),
                        'rss_GiB': (vals[:, 2] / 1024).tolist(),
                        'vms_GiB': (vals[:, 3] / 1024).tolist(),
                    }
            os.chdir(syscwd)

        return results

    def _list_outputs(self):
        """ List the expected outputs
        """
        if self.output_spec:
            raise NotImplementedError
        else:
            return None

    def aggregate_outputs(self, runtime=None, needed_outputs=None):
        """Collate expected outputs and apply output traits validation."""
        outputs = self._outputs()  # Generate an empty output spec object
        predicted_outputs = self._list_outputs()  # Predictions from _list_outputs
        if not predicted_outputs:
            return outputs

        # Precalculate the list of output trait names that should be aggregated
        aggregate_names = set(predicted_outputs)
        if needed_outputs is not None:
            aggregate_names = set(needed_outputs).intersection(aggregate_names)

        if aggregate_names:  # Make sure outputs are compatible
            _na_outputs = self._check_version_requirements(outputs)
            na_names = aggregate_names.intersection(_na_outputs)
            if na_names:
                # XXX Change to TypeError in Nipype 2.0
                raise KeyError("""\
Output trait(s) %s not available in version %s of interface %s.\
""" % (', '.join(na_names), self.version, self.__class__.__name__))

        for key in aggregate_names:  # Final aggregation
            val = predicted_outputs[key]
            try:
                setattr(outputs, key, val)
            except TraitError as error:
                if 'an existing' in getattr(error, 'info', 'default'):
                    msg = "No such file or directory '%s' for output '%s' of a %s interface" % \
                        (val, key, self.__class__.__name__)
                    raise FileNotFoundError(msg)
                raise error
        return outputs

    @property
    def version(self):
        if self._version is None:
            if str2bool(config.get('execution', 'stop_on_unknown_version')):
                raise ValueError('Interface %s has no version information' %
                                 self.__class__.__name__)
        return self._version

    def load_inputs_from_json(self, json_file, overwrite=True):
        """
        A convenient way to load pre-set inputs from a JSON file.
        """

        with open(json_file) as fhandle:
            inputs_dict = json.load(fhandle)

        def_inputs = []
        if not overwrite:
            def_inputs = list(self.inputs.get_traitsfree().keys())

        new_inputs = list(set(list(inputs_dict.keys())) - set(def_inputs))
        for key in new_inputs:
            if hasattr(self.inputs, key):
                setattr(self.inputs, key, inputs_dict[key])

    def save_inputs_to_json(self, json_file):
        """
        A convenient way to save current inputs to a JSON file.
        """
        inputs = self.inputs.get_traitsfree()
        iflogger.debug('saving inputs %s', inputs)
        with open(json_file, 'w' if PY3 else 'wb') as fhandle:
            json.dump(inputs, fhandle, indent=4, ensure_ascii=False)

    def _pre_run_hook(self, runtime):
        """
        Perform any pre-_run_interface() processing

        Subclasses may override this function to modify ``runtime`` object or
        interface state

        MUST return runtime object
        """
        return runtime

    def _post_run_hook(self, runtime):
        """
        Perform any post-_run_interface() processing

        Subclasses may override this function to modify ``runtime`` object or
        interface state

        MUST return runtime object
        """
        return runtime


class SimpleInterface(BaseInterface):
    """ An interface pattern that allows outputs to be set in a dictionary
    called ``_results`` that is automatically interpreted by
    ``_list_outputs()`` to find the outputs.

    When implementing ``_run_interface``, set outputs with::

        self._results[out_name] = out_value

    This can be a way to upgrade a ``Function`` interface to do type checking.

    Examples
    --------

    >>> from nipype.interfaces.base import (
    ...     SimpleInterface, BaseInterfaceInputSpec, TraitedSpec)

    >>> def double(x):
    ...    return 2 * x
    ...
    >>> class DoubleInputSpec(BaseInterfaceInputSpec):
    ...     x = traits.Float(mandatory=True)
    ...
    >>> class DoubleOutputSpec(TraitedSpec):
    ...     doubled = traits.Float()
    ...
    >>> class Double(SimpleInterface):
    ...     input_spec = DoubleInputSpec
    ...     output_spec = DoubleOutputSpec
    ...
    ...     def _run_interface(self, runtime):
    ...          self._results['doubled'] = double(self.inputs.x)
    ...          return runtime

    >>> dbl = Double()
    >>> dbl.inputs.x = 2
    >>> dbl.run().outputs.doubled
    4.0
    """

    def __init__(self, from_file=None, resource_monitor=None, **inputs):
        super(SimpleInterface, self).__init__(
            from_file=from_file, resource_monitor=resource_monitor, **inputs)
        self._results = {}

    def _list_outputs(self):
        return self._results


class CommandLine(BaseInterface):
    """Implements functionality to interact with command line programs
    class must be instantiated with a command argument

    Parameters
    ----------

    command : string
        define base immutable `command` you wish to run

    args : string, optional
        optional arguments passed to base `command`


    Examples
    --------
    >>> import pprint
    >>> from nipype.interfaces.base import CommandLine
    >>> cli = CommandLine(command='ls', environ={'DISPLAY': ':1'})
    >>> cli.inputs.args = '-al'
    >>> cli.cmdline
    'ls -al'

    # Use get_traitsfree() to check all inputs set
    >>> pprint.pprint(cli.inputs.get_traitsfree())  # doctest:
    {'args': '-al',
     'environ': {'DISPLAY': ':1'}}

    >>> cli.inputs.get_hashval()[0][0]
    ('args', '-al')
    >>> cli.inputs.get_hashval()[1]
    '11c37f97649cd61627f4afe5136af8c0'

    """
    input_spec = CommandLineInputSpec
    _cmd_prefix = ''
    _cmd = None
    _version = None
    _terminal_output = 'stream'

    @classmethod
    def set_default_terminal_output(cls, output_type):
        """Set the default terminal output for CommandLine Interfaces.

        This method is used to set default terminal output for
        CommandLine Interfaces.  However, setting this will not
        update the output type for any existing instances.  For these,
        assign the <instance>.terminal_output.
        """

        if output_type in VALID_TERMINAL_OUTPUT:
            cls._terminal_output = output_type
        else:
            raise AttributeError(
                'Invalid terminal output_type: %s' % output_type)

    def __init__(self, command=None, terminal_output=None, **inputs):
        super(CommandLine, self).__init__(**inputs)
        self._environ = None
        # Set command. Input argument takes precedence
        self._cmd = command or getattr(self, '_cmd', None)

        # Store dependencies in runtime object
        self._ldd = str2bool(
            config.get('execution', 'get_linked_libs', 'true'))

        if self._cmd is None:
            raise Exception("Missing command")

        if terminal_output is not None:
            self.terminal_output = terminal_output

    @property
    def cmd(self):
        """sets base command, immutable"""
        if not self._cmd:
            raise NotImplementedError(
                'CommandLineInterface should wrap an executable, but '
                'none has been set.')
        return self._cmd

    @property
    def cmdline(self):
        """ `command` plus any arguments (args)
        validates arguments and generates command line"""
        self._check_mandatory_inputs()
        allargs = [self._cmd_prefix + self.cmd] + self._parse_inputs()
        return ' '.join(allargs)

    @property
    def terminal_output(self):
        return self._terminal_output

    @terminal_output.setter
    def terminal_output(self, value):
        if value not in VALID_TERMINAL_OUTPUT:
            raise RuntimeError(
                'Setting invalid value "%s" for terminal_output. Valid values are '
                '%s.' % (value,
                         ', '.join(['"%s"' % v
                                    for v in VALID_TERMINAL_OUTPUT])))
        self._terminal_output = value

    def raise_exception(self, runtime):
        raise RuntimeError(
            ('Command:\n{cmdline}\nStandard output:\n{stdout}\n'
             'Standard error:\n{stderr}\nReturn code: {returncode}'
             ).format(**runtime.dictcopy()))

    def _get_environ(self):
        return getattr(self.inputs, 'environ', {})

    def version_from_command(self, flag='-v', cmd=None):
        iflogger.warning('version_from_command member of CommandLine was '
                         'Deprecated in nipype-1.0.0 and deleted in 1.1.0')
        if cmd is None:
            cmd = self.cmd.split()[0]

        env = dict(os.environ)
        if which(cmd, env=env):
            out_environ = self._get_environ()
            env.update(out_environ)
            proc = sp.Popen(
                ' '.join((cmd, flag)),
                shell=True,
                env=env,
                stdout=sp.PIPE,
                stderr=sp.PIPE,
            )
            o, e = proc.communicate()
            return o

    def _run_interface(self, runtime, correct_return_codes=(0, )):
        """Execute command via subprocess

        Parameters
        ----------
        runtime : passed by the run function

        Returns
        -------
        runtime : updated runtime information
            adds stdout, stderr, merged, cmdline, dependencies, command_path

        """

        out_environ = self._get_environ()
        # Initialize runtime Bunch
        runtime.stdout = None
        runtime.stderr = None
        runtime.cmdline = self.cmdline
        runtime.environ.update(out_environ)

        # which $cmd
        executable_name = shlex.split(self._cmd_prefix + self.cmd)[0]
        cmd_path = which(executable_name, env=runtime.environ)

        if cmd_path is None:
            raise IOError(
                'No command "%s" found on host %s. Please check that the '
                'corresponding package is installed.' % (executable_name,
                                                         runtime.hostname))

        runtime.command_path = cmd_path
        runtime.dependencies = (get_dependencies(executable_name,
                                                 runtime.environ)
                                if self._ldd else '<skipped>')
        runtime = run_command(runtime, output=self.terminal_output)
        if runtime.returncode is None or \
                runtime.returncode not in correct_return_codes:
            self.raise_exception(runtime)

        return runtime

    def _format_arg(self, name, trait_spec, value):
        """A helper function for _parse_inputs

        Formats a trait containing argstr metadata
        """
        argstr = trait_spec.argstr
        iflogger.debug('%s_%s', name, value)
        if trait_spec.is_trait_type(traits.Bool) and "%" not in argstr:
            # Boolean options have no format string. Just append options if True.
            return argstr if value else None
        # traits.Either turns into traits.TraitCompound and does not have any
        # inner_traits
        elif trait_spec.is_trait_type(traits.List) \
            or (trait_spec.is_trait_type(traits.TraitCompound) and
                isinstance(value, list)):
            # This is a bit simple-minded at present, and should be
            # construed as the default. If more sophisticated behavior
            # is needed, it can be accomplished with metadata (e.g.
            # format string for list member str'ification, specifying
            # the separator, etc.)

            # Depending on whether we stick with traitlets, and whether or
            # not we beef up traitlets.List, we may want to put some
            # type-checking code here as well
            sep = trait_spec.sep if trait_spec.sep is not None else ' '

            if argstr.endswith('...'):
                # repeatable option
                # --id %d... will expand to
                # --id 1 --id 2 --id 3 etc.,.
                argstr = argstr.replace('...', '')
                return sep.join([argstr % elt for elt in value])
            else:
                return argstr % sep.join(str(elt) for elt in value)
        else:
            # Append options using format string.
            return argstr % value

    def _filename_from_source(self, name, chain=None):
        if chain is None:
            chain = []

        trait_spec = self.inputs.trait(name)
        retval = getattr(self.inputs, name)
        source_ext = None
        if not isdefined(retval) or "%s" in retval:
            if not trait_spec.name_source:
                return retval

            # Do not generate filename when excluded by other inputs
            if any(isdefined(getattr(self.inputs, field))
                   for field in trait_spec.xor or ()):
                return retval

            # Do not generate filename when required fields are missing
            if not all(isdefined(getattr(self.inputs, field))
                       for field in trait_spec.requires or ()):
                return retval

            if isdefined(retval) and "%s" in retval:
                name_template = retval
            else:
                name_template = trait_spec.name_template
            if not name_template:
                name_template = "%s_generated"

            ns = trait_spec.name_source
            while isinstance(ns, (list, tuple)):
                if len(ns) > 1:
                    iflogger.warning(
                        'Only one name_source per trait is allowed')
                ns = ns[0]

            if not isinstance(ns, (str, bytes)):
                raise ValueError(
                    'name_source of \'{}\' trait should be an input trait '
                    'name, but a type {} object was found'.format(
                        name, type(ns)))

            if isdefined(getattr(self.inputs, ns)):
                name_source = ns
                source = getattr(self.inputs, name_source)
                while isinstance(source, list):
                    source = source[0]

                # special treatment for files
                try:
                    _, base, source_ext = split_filename(source)
                except (AttributeError, TypeError):
                    base = source
            else:
                if name in chain:
                    raise NipypeInterfaceError(
                        'Mutually pointing name_sources')

                chain.append(name)
                base = self._filename_from_source(ns, chain)
                if isdefined(base):
                    _, _, source_ext = split_filename(base)
                else:
                    # Do not generate filename when required fields are missing
                    return retval

            chain = None
            retval = name_template % base
            _, _, ext = split_filename(retval)
            if trait_spec.keep_extension and (ext or source_ext):
                if (ext is None or not ext) and source_ext:
                    retval = retval + source_ext
            else:
                retval = self._overload_extension(retval, name)
        return retval

    def _gen_filename(self, name):
        raise NotImplementedError

    def _overload_extension(self, value, name=None):
        return value

    def _list_outputs(self):
        metadata = dict(name_source=lambda t: t is not None)
        traits = self.inputs.traits(**metadata)
        if traits:
            outputs = self.output_spec().trait_get()
            for name, trait_spec in list(traits.items()):
                out_name = name
                if trait_spec.output_name is not None:
                    out_name = trait_spec.output_name
                fname = self._filename_from_source(name)
                if isdefined(fname):
                    outputs[out_name] = os.path.abspath(fname)
            return outputs

    def _parse_inputs(self, skip=None):
        """Parse all inputs using the ``argstr`` format string in the Trait.

        Any inputs that are assigned (not the default_value) are formatted
        to be added to the command line.

        Returns
        -------
        all_args : list
            A list of all inputs formatted for the command line.

        """
        all_args = []
        initial_args = {}
        final_args = {}
        metadata = dict(argstr=lambda t: t is not None)
        for name, spec in sorted(self.inputs.traits(**metadata).items()):
            if skip and name in skip:
                continue
            value = getattr(self.inputs, name)
            if spec.name_source:
                value = self._filename_from_source(name)
            elif spec.genfile:
                if not isdefined(value) or value is None:
                    value = self._gen_filename(name)

            if not isdefined(value):
                continue
            arg = self._format_arg(name, spec, value)
            if arg is None:
                continue
            pos = spec.position
            if pos is not None:
                if int(pos) >= 0:
                    initial_args[pos] = arg
                else:
                    final_args[pos] = arg
            else:
                all_args.append(arg)
        first_args = [el for _, el in sorted(initial_args.items())]
        last_args = [el for _, el in sorted(final_args.items())]
        return first_args + all_args + last_args


class StdOutCommandLine(CommandLine):
    input_spec = StdOutCommandLineInputSpec

    def _gen_filename(self, name):
        return self._gen_outfilename() if name == 'out_file' else None

    def _gen_outfilename(self):
        raise NotImplementedError


class MpiCommandLine(CommandLine):
    """Implements functionality to interact with command line programs
    that can be run with MPI (i.e. using 'mpiexec').

    Examples
    --------
    >>> from nipype.interfaces.base import MpiCommandLine
    >>> mpi_cli = MpiCommandLine(command='my_mpi_prog')
    >>> mpi_cli.inputs.args = '-v'
    >>> mpi_cli.cmdline
    'my_mpi_prog -v'

    >>> mpi_cli.inputs.use_mpi = True
    >>> mpi_cli.inputs.n_procs = 8
    >>> mpi_cli.cmdline
    'mpiexec -n 8 my_mpi_prog -v'
    """
    input_spec = MpiCommandLineInputSpec

    @property
    def cmdline(self):
        """Adds 'mpiexec' to begining of command"""
        result = []
        if self.inputs.use_mpi:
            result.append('mpiexec')
            if self.inputs.n_procs:
                result.append('-n %d' % self.inputs.n_procs)
        result.append(super(MpiCommandLine, self).cmdline)
        return ' '.join(result)


class SEMLikeCommandLine(CommandLine):
    """In SEM derived interface all outputs have corresponding inputs.
    However, some SEM commands create outputs that are not defined in the XML.
    In those cases one has to create a subclass of the autogenerated one and
    overload the _list_outputs method. _outputs_from_inputs should still be
    used but only for the reduced (by excluding those that do not have
    corresponding inputs list of outputs.
    """

    def _list_outputs(self):
        outputs = self.output_spec().trait_get()
        return self._outputs_from_inputs(outputs)

    def _outputs_from_inputs(self, outputs):
        for name in list(outputs.keys()):
            corresponding_input = getattr(self.inputs, name)
            if isdefined(corresponding_input):
                if (isinstance(corresponding_input, bool)
                        and corresponding_input):
                    outputs[name] = \
                        os.path.abspath(self._outputs_filenames[name])
                else:
                    if isinstance(corresponding_input, list):
                        outputs[name] = [
                            os.path.abspath(inp) for inp in corresponding_input
                        ]
                    else:
                        outputs[name] = os.path.abspath(corresponding_input)
        return outputs

    def _format_arg(self, name, spec, value):
        if name in list(self._outputs_filenames.keys()):
            if isinstance(value, bool):
                if value:
                    value = os.path.abspath(self._outputs_filenames[name])
                else:
                    return ""
        return super(SEMLikeCommandLine, self)._format_arg(name, spec, value)


class LibraryBaseInterface(BaseInterface):
    _pkg = None
    imports = ()

    def __init__(self, check_import=True, *args, **kwargs):
        super(LibraryBaseInterface, self).__init__(*args, **kwargs)
        if check_import:
            import pkgutil
            failed_imports = []
            for pkg in (self._pkg,) + tuple(self.imports):
                if pkgutil.find_loader(pkg) is None:
                    failed_imports.append(pkg)
            if failed_imports:
                iflogger.warning('Unable to import %s; %s interface may fail to '
                                 'run', failed_imports, self.__class__.__name__)

    @property
    def version(self):
        if self._version is None:
            import importlib
            try:
                self._version = importlib.import_module(self._pkg).__version__
            except (ImportError, AttributeError):
                pass
        return super(LibraryBaseInterface, self).version


class PackageInfo(object):
    _version = None
    version_cmd = None
    version_file = None

    @classmethod
    def version(klass):
        if klass._version is None:
            if klass.version_cmd is not None:
                try:
                    clout = CommandLine(
                        command=klass.version_cmd,
                        resource_monitor=False,
                        terminal_output='allatonce').run()
                except IOError:
                    return None

                raw_info = clout.runtime.stdout
            elif klass.version_file is not None:
                try:
                    with open(klass.version_file, 'rt') as fobj:
                        raw_info = fobj.read()
                except OSError:
                    return None
            else:
                return None

            klass._version = klass.parse_version(raw_info)

        return klass._version

    @staticmethod
    def parse_version(raw_info):
        raise NotImplementedError



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import FilterRegressor


def test_FilterRegressor_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        design_file=dict(
            argstr='-d %s',
            extensions=None,
            mandatory=True,
            position=3,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        filter_all=dict(
            argstr="-f '%s'",
            mandatory=True,
            position=4,
            xor=['filter_columns'],
        ),
        filter_columns=dict(
            argstr="-f '%s'",
            mandatory=True,
            position=4,
            xor=['filter_all'],
        ),
        in_file=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        mask=dict(
            argstr='-m %s',
            extensions=None,
        ),
        out_file=dict(
            argstr='-o %s',
            extensions=None,
            genfile=True,
            hash_files=False,
            position=2,
        ),
        out_vnscales=dict(argstr='--out_vnscales', ),
        output_type=dict(),
        var_norm=dict(argstr='--vn', ),
    )
    inputs = FilterRegressor.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FilterRegressor_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = FilterRegressor.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brainsuite import BDP


def test_BDP_inputs():
    input_map = dict(
        BVecBValPair=dict(
            argstr='--bvec %s --bval %s',
            mandatory=True,
            position=-1,
            xor=['bMatrixFile'],
        ),
        args=dict(argstr='%s', ),
        bMatrixFile=dict(
            argstr='--bmat %s',
            extensions=None,
            mandatory=True,
            position=-1,
            xor=['BVecBValPair'],
        ),
        bValRatioThreshold=dict(argstr='--bval-ratio-threshold %f', ),
        bfcFile=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=0,
            xor=['noStructuralRegistration'],
        ),
        customDiffusionLabel=dict(
            argstr='--custom-diffusion-label %s',
            extensions=None,
        ),
        customLabelXML=dict(
            argstr='--custom-label-xml %s',
            extensions=None,
        ),
        customT1Label=dict(
            argstr='--custom-t1-label %s',
            extensions=None,
        ),
        dataSinkDelay=dict(argstr='%s', ),
        dcorrRegMeasure=dict(argstr='--dcorr-reg-method %s', ),
        dcorrWeight=dict(argstr='--dcorr-regularization-wt %f', ),
        dwiMask=dict(
            argstr='--dwi-mask %s',
            extensions=None,
        ),
        echoSpacing=dict(argstr='--echo-spacing=%f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        estimateODF_3DShore=dict(argstr='--3dshore --diffusion_time_ms %f', ),
        estimateODF_FRACT=dict(argstr='--FRACT', ),
        estimateODF_FRT=dict(argstr='--FRT', ),
        estimateTensors=dict(argstr='--tensors', ),
        fieldmapCorrection=dict(
            argstr='--fieldmap-correction %s',
            extensions=None,
            requires=['echoSpacing'],
        ),
        fieldmapCorrectionMethod=dict(
            argstr='--fieldmap-correction-method %s',
            xor=['skipIntensityCorr'],
        ),
        fieldmapSmooth=dict(argstr='--fieldmap-smooth3=%f', ),
        flagConfigFile=dict(
            argstr='--flag-conf-file %s',
            extensions=None,
        ),
        forcePartialROIStats=dict(argstr='--force-partial-roi-stats', ),
        generateStats=dict(argstr='--generate-stats', ),
        ignoreFieldmapFOV=dict(argstr='--ignore-fieldmap-fov', ),
        ignoreMemory=dict(argstr='--ignore-memory', ),
        inputDiffusionData=dict(
            argstr='--nii %s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        lowMemory=dict(argstr='--low-memory', ),
        noStructuralRegistration=dict(
            argstr='--no-structural-registration',
            mandatory=True,
            position=0,
            xor=['bfcFile'],
        ),
        odfLambta=dict(argstr='--odf-lambda <L>', ),
        onlyStats=dict(argstr='--generate-only-stats', ),
        outPrefix=dict(argstr='--output-fileprefix %s', ),
        outputDiffusionCoordinates=dict(
            argstr='--output-diffusion-coordinate', ),
        outputSubdir=dict(argstr='--output-subdir %s', ),
        phaseEncodingDirection=dict(argstr='--dir=%s', ),
        rigidRegMeasure=dict(argstr='--rigid-reg-measure %s', ),
        skipDistortionCorr=dict(argstr='--no-distortion-correction', ),
        skipIntensityCorr=dict(
            argstr='--no-intensity-correction',
            xor=['fieldmapCorrectionMethod'],
        ),
        skipNonuniformityCorr=dict(argstr='--no-nonuniformity-correction', ),
        t1Mask=dict(
            argstr='--t1-mask %s',
            extensions=None,
        ),
        threads=dict(argstr='--threads=%d', ),
        transformDataOnly=dict(argstr='--transform-data-only', ),
        transformDiffusionSurface=dict(
            argstr='--transform-diffusion-surface %s',
            extensions=None,
        ),
        transformDiffusionVolume=dict(
            argstr='--transform-diffusion-volume %s',
            extensions=None,
        ),
        transformInterpolation=dict(argstr='--transform-interpolation %s', ),
        transformT1Surface=dict(
            argstr='--transform-t1-surface %s',
            extensions=None,
        ),
        transformT1Volume=dict(
            argstr='--transform-t1-volume %s',
            extensions=None,
        ),
    )
    inputs = BDP.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value



'''
Created on Aug 2, 2013

@author: chadcumba

Parallel workflow execution with SLURM
'''
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import open

import os
import re
from time import sleep

from ... import logging
from ...interfaces.base import CommandLine
from .base import SGELikeBatchManagerBase, logger

iflogger = logging.getLogger('nipype.interface')


class SLURMPlugin(SGELikeBatchManagerBase):
    '''
    Execute using SLURM

    The plugin_args input to run can be used to control the SLURM execution.
    Currently supported options are:

    - template : template to use for batch job submission

    - sbatch_args: arguments to pass prepend to the sbatch call


    '''

    def __init__(self, **kwargs):

        template = "#!/bin/bash"

        self._retry_timeout = 2
        self._max_tries = 2
        self._template = template
        self._sbatch_args = None
        self._jobid_re = "Submitted batch job ([0-9]*)"

        if 'plugin_args' in kwargs and kwargs['plugin_args']:
            if 'retry_timeout' in kwargs['plugin_args']:
                self._retry_timeout = kwargs['plugin_args']['retry_timeout']
            if 'max_tries' in kwargs['plugin_args']:
                self._max_tries = kwargs['plugin_args']['max_tries']
            if 'jobid_re' in kwargs['plugin_args']:
                self._jobid_re = kwargs['plugin_args']['jobid_re']
            if 'template' in kwargs['plugin_args']:
                self._template = kwargs['plugin_args']['template']
                if os.path.isfile(self._template):
                    with open(self._template) as f:
                        self._template = f.read()
            if 'sbatch_args' in kwargs['plugin_args']:
                self._sbatch_args = kwargs['plugin_args']['sbatch_args']
        self._pending = {}
        super(SLURMPlugin, self).__init__(self._template, **kwargs)

    def _is_pending(self, taskid):
        try:
            res = CommandLine(
                'squeue',
                args=' '.join(['-j', '%s' % taskid]),
                resource_monitor=False,
                terminal_output='allatonce').run()
            return res.runtime.stdout.find(str(taskid)) > -1
        except RuntimeError as e:
            if any(ss in str(e) for ss
                   in ['Socket timed out', 'not available at the moment']):
                # do not raise error and allow recheck
                logger.warning(
                    "SLURM timeout encountered while checking job status,"
                    " treating job %d as pending", taskid
                )
                return True
            if 'Invalid job id' not in str(e):
                raise(e)
            return False

    def _submit_batchtask(self, scriptfile, node):
        """
        This is more or less the _submit_batchtask from sge.py with flipped
        variable names, different command line switches, and different output
        formatting/processing
        """
        cmd = CommandLine(
            'sbatch',
            environ=dict(os.environ),
            resource_monitor=False,
            terminal_output='allatonce')
        path = os.path.dirname(scriptfile)

        sbatch_args = ''
        if self._sbatch_args:
            sbatch_args = self._sbatch_args
        if 'sbatch_args' in node.plugin_args:
            if 'overwrite' in node.plugin_args and\
               node.plugin_args['overwrite']:
                sbatch_args = node.plugin_args['sbatch_args']
            else:
                sbatch_args += (" " + node.plugin_args['sbatch_args'])
        if '-o' not in sbatch_args:
            sbatch_args = '%s -o %s' % (sbatch_args,
                                        os.path.join(path, 'slurm-%j.out'))
        if '-e' not in sbatch_args:
            sbatch_args = '%s -e %s' % (sbatch_args,
                                        os.path.join(path, 'slurm-%j.out'))
        if node._hierarchy:
            jobname = '.'.join((dict(os.environ)['LOGNAME'], node._hierarchy,
                                node._id))
        else:
            jobname = '.'.join((dict(os.environ)['LOGNAME'], node._id))
        jobnameitems = jobname.split('.')
        jobnameitems.reverse()
        jobname = '.'.join(jobnameitems)
        cmd.inputs.args = '%s -J %s %s' % (sbatch_args, jobname, scriptfile)
        oldlevel = iflogger.level
        iflogger.setLevel(logging.getLevelName('CRITICAL'))
        tries = 0
        while True:
            try:
                result = cmd.run()
            except Exception as e:
                if tries < self._max_tries:
                    tries += 1
                    # sleep 2 seconds and try again.
                    sleep(self._retry_timeout)
                else:
                    iflogger.setLevel(oldlevel)
                    raise RuntimeError('\n'.join(
                        (('Could not submit sbatch task'
                          ' for node %s') % node._id, str(e))))
            else:
                break
        logger.debug('Ran command ({0})'.format(cmd.cmdline))
        iflogger.setLevel(oldlevel)
        # retrieve taskid
        lines = [line for line in result.runtime.stdout.split('\n') if line]
        taskid = int(re.match(self._jobid_re, lines[-1]).groups()[0])
        self._pending[taskid] = node.output_dir()
        logger.debug('submitted sbatch task: %d for node %s' % (taskid,
                                                                node._id))
        return taskid



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..registration import Registration


def test_Registration_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixed_image=dict(
            argstr='-f %s',
            extensions=None,
            mandatory=True,
        ),
        fixed_mask=dict(
            argstr='-fMask %s',
            extensions=None,
        ),
        initial_transform=dict(
            argstr='-t0 %s',
            extensions=None,
        ),
        moving_image=dict(
            argstr='-m %s',
            extensions=None,
            mandatory=True,
        ),
        moving_mask=dict(
            argstr='-mMask %s',
            extensions=None,
        ),
        num_threads=dict(
            argstr='-threads %01d',
            nohash=True,
            usedefault=True,
        ),
        output_path=dict(
            argstr='-out %s',
            mandatory=True,
            usedefault=True,
        ),
        parameters=dict(
            argstr='-p %s...',
            mandatory=True,
        ),
    )
    inputs = Registration.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Registration_outputs():
    output_map = dict(
        transform=dict(),
        warped_file=dict(extensions=None, ),
        warped_files=dict(),
        warped_files_flags=dict(),
    )
    outputs = Registration.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import CommandLine, CommandLineInputSpec, SEMLikeCommandLine, TraitedSpec, File, Directory, traits, isdefined, InputMultiPath, OutputMultiPath
import os


class GrayscaleGrindPeakImageFilterInputSpec(CommandLineInputSpec):
    inputVolume = File(
        position=-2,
        desc="Input volume to be filtered",
        exists=True,
        argstr="%s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output filtered",
        argstr="%s")


class GrayscaleGrindPeakImageFilterOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Output filtered", exists=True)


class GrayscaleGrindPeakImageFilter(SEMLikeCommandLine):
    """title: Grayscale Grind Peak Image Filter

category: Filtering.Morphology

description: GrayscaleGrindPeakImageFilter removes peaks in a grayscale image. Peaks are local maxima in the grayscale topography that are not connected to boundaries of the image. Gray level values adjacent to a peak are extrapolated through the peak.

This filter is used to smooth over local maxima without affecting the values of local minima.  If you take the difference between the output of this filter and the original image (and perhaps threshold the difference above a small value), you'll obtain a map of the local maxima.

This filter uses the GrayscaleGeodesicDilateImageFilter.  It provides its own input as the "mask" input to the geodesic erosion.  The "marker" image for the geodesic erosion is constructed such that boundary pixels match the boundary pixels of the input image and the interior pixels are set to the minimum pixel value in the input image.

This filter is the dual to the GrayscaleFillholeImageFilter which implements the Fillhole algorithm.  Since it is a dual, it is somewhat superfluous but is provided as a convenience.

Geodesic morphology and the Fillhole algorithm is described in Chapter 6 of Pierre Soille's book "Morphological Image Analysis: Principles and Applications", Second Edition, Springer, 2003.

A companion filter, Grayscale Fill Hole, fills holes in grayscale images.

version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/GrayscaleGrindPeakImageFilter

contributor: Bill Lorensen (GE)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = GrayscaleGrindPeakImageFilterInputSpec
    output_spec = GrayscaleGrindPeakImageFilterOutputSpec
    _cmd = "GrayscaleGrindPeakImageFilter "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii'}


class GrayscaleFillHoleImageFilterInputSpec(CommandLineInputSpec):
    inputVolume = File(
        position=-2,
        desc="Input volume to be filtered",
        exists=True,
        argstr="%s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output filtered",
        argstr="%s")


class GrayscaleFillHoleImageFilterOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Output filtered", exists=True)


class GrayscaleFillHoleImageFilter(SEMLikeCommandLine):
    """title: Grayscale Fill Hole Image Filter

category: Filtering.Morphology

description: GrayscaleFillholeImageFilter fills holes in a grayscale image.  Holes are local minima in the grayscale topography that are not connected to boundaries of the image. Gray level values adjacent to a hole are extrapolated across the hole.

This filter is used to smooth over local minima without affecting the values of local maxima.  If you take the difference between the output of this filter and the original image (and perhaps threshold the difference above a small value), you'll obtain a map of the local minima.

This filter uses the itkGrayscaleGeodesicErodeImageFilter.  It provides its own input as the "mask" input to the geodesic erosion.  The "marker" image for the geodesic erosion is constructed such that boundary pixels match the boundary pixels of the input image and the interior pixels are set to the maximum pixel value in the input image.

 Geodesic morphology and the Fillhole algorithm is described in Chapter 6 of Pierre Soille's book "Morphological Image Analysis: Principles and Applications", Second Edition, Springer, 2003.

 A companion filter, Grayscale Grind Peak, removes peaks in grayscale images.

version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/GrayscaleFillHoleImageFilter

contributor: Bill Lorensen (GE)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = GrayscaleFillHoleImageFilterInputSpec
    output_spec = GrayscaleFillHoleImageFilterOutputSpec
    _cmd = "GrayscaleFillHoleImageFilter "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii'}



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import CreateWarped


def test_CreateWarped_inputs():
    input_map = dict(
        flowfield_files=dict(
            copyfile=False,
            field='crt_warped.flowfields',
            mandatory=True,
        ),
        image_files=dict(
            copyfile=False,
            field='crt_warped.images',
            mandatory=True,
        ),
        interp=dict(field='crt_warped.interp', ),
        iterations=dict(field='crt_warped.K', ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        modulate=dict(field='crt_warped.jactransf', ),
        paths=dict(),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
    )
    inputs = CreateWarped.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_CreateWarped_outputs():
    output_map = dict(warped_files=dict(), )
    outputs = CreateWarped.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import TProject


def test_TProject_inputs():
    input_map = dict(
        TR=dict(argstr='-TR %g', ),
        args=dict(argstr='%s', ),
        automask=dict(
            argstr='-automask',
            xor=['mask'],
        ),
        bandpass=dict(argstr='-bandpass %g %g', ),
        blur=dict(argstr='-blur %g', ),
        cenmode=dict(argstr='-cenmode %s', ),
        censor=dict(
            argstr='-censor %s',
            extensions=None,
        ),
        censortr=dict(argstr='-CENSORTR %s', ),
        concat=dict(
            argstr='-concat %s',
            extensions=None,
        ),
        dsort=dict(argstr='-dsort %s...', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-input %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=1,
        ),
        mask=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        noblock=dict(argstr='-noblock', ),
        norm=dict(argstr='-norm', ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        ort=dict(
            argstr='-ort %s',
            extensions=None,
        ),
        out_file=dict(
            argstr='-prefix %s',
            extensions=None,
            name_source='in_file',
            name_template='%s_tproject',
            position=-1,
        ),
        outputtype=dict(),
        polort=dict(argstr='-polort %d', ),
        stopband=dict(argstr='-stopband %g %g', ),
    )
    inputs = TProject.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_TProject_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = TProject.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..modelgen import SpecifyModel


def test_SpecifyModel_inputs():
    input_map = dict(
        bids_amplitude_column=dict(),
        bids_condition_column=dict(usedefault=True, ),
        bids_event_file=dict(
            mandatory=True,
            xor=['subject_info', 'event_files', 'bids_event_file'],
        ),
        event_files=dict(
            mandatory=True,
            xor=['subject_info', 'event_files', 'bids_event_file'],
        ),
        functional_runs=dict(
            copyfile=False,
            mandatory=True,
        ),
        high_pass_filter_cutoff=dict(mandatory=True, ),
        input_units=dict(mandatory=True, ),
        outlier_files=dict(copyfile=False, ),
        parameter_source=dict(usedefault=True, ),
        realignment_parameters=dict(copyfile=False, ),
        subject_info=dict(
            mandatory=True,
            xor=['subject_info', 'event_files', 'bids_event_file'],
        ),
        time_repetition=dict(mandatory=True, ),
    )
    inputs = SpecifyModel.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_SpecifyModel_outputs():
    output_map = dict(session_info=dict(), )
    outputs = SpecifyModel.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import str, bytes

import os.path as op
import glob

from ... import logging
from ...utils.filemanip import simplify_list
from ..base import (traits, File, Directory, TraitedSpec, OutputMultiPath)
from ..freesurfer.base import FSCommand, FSTraitedSpec

iflogger = logging.getLogger('nipype.interface')


class WatershedBEMInputSpec(FSTraitedSpec):
    subject_id = traits.Str(
        argstr='--subject %s',
        mandatory=True,
        desc='Subject ID (must have a complete Freesurfer directory)')
    subjects_dir = Directory(
        exists=True,
        mandatory=True,
        usedefault=True,
        desc='Path to Freesurfer subjects directory')
    volume = traits.Enum(
        'T1',
        'aparc+aseg',
        'aseg',
        'brain',
        'orig',
        'brainmask',
        'ribbon',
        argstr='--volume %s',
        usedefault=True,
        desc='The volume from the "mri" directory to use (defaults to T1)')
    overwrite = traits.Bool(
        True,
        usedefault=True,
        argstr='--overwrite',
        desc='Overwrites the existing files')
    atlas_mode = traits.Bool(
        argstr='--atlas',
        desc='Use atlas mode for registration (default: no rigid alignment)')


class WatershedBEMOutputSpec(TraitedSpec):
    mesh_files = OutputMultiPath(
        File(exists=True),
        desc=('Paths to the output meshes (brain, inner '
              'skull, outer skull, outer skin)'))
    brain_surface = File(
        exists=True,
        loc='bem/watershed',
        desc='Brain surface (in Freesurfer format)')
    inner_skull_surface = File(
        exists=True,
        loc='bem/watershed',
        desc='Inner skull surface (in Freesurfer format)')
    outer_skull_surface = File(
        exists=True,
        loc='bem/watershed',
        desc='Outer skull surface (in Freesurfer format)')
    outer_skin_surface = File(
        exists=True,
        loc='bem/watershed',
        desc='Outer skin surface (in Freesurfer format)')
    fif_file = File(
        exists=True,
        loc='bem',
        altkey='fif',
        desc='"fif" format file for EEG processing in MNE')
    cor_files = OutputMultiPath(
        File(exists=True),
        loc='bem/watershed/ws',
        altkey='COR',
        desc='"COR" format files')


class WatershedBEM(FSCommand):
    """Uses mne_watershed_bem to get information from dicom directories

    Examples
    --------

    >>> from nipype.interfaces.mne import WatershedBEM
    >>> bem = WatershedBEM()
    >>> bem.inputs.subject_id = 'subj1'
    >>> bem.inputs.subjects_dir = '.'
    >>> bem.cmdline
    'mne watershed_bem --overwrite --subject subj1 --volume T1'
    >>> bem.run() 				# doctest: +SKIP

   """

    _cmd = 'mne watershed_bem'
    input_spec = WatershedBEMInputSpec
    output_spec = WatershedBEMOutputSpec
    _additional_metadata = ['loc', 'altkey']

    def _get_files(self, path, key, dirval, altkey=None):
        globsuffix = '*'
        globprefix = '*'
        keydir = op.join(path, dirval)
        if altkey:
            key = altkey
        globpattern = op.join(keydir, ''.join((globprefix, key, globsuffix)))
        return glob.glob(globpattern)

    def _list_outputs(self):
        outputs = self.output_spec().get()
        subjects_dir = self.inputs.subjects_dir
        subject_path = op.join(subjects_dir, self.inputs.subject_id)
        output_traits = self._outputs()
        mesh_paths = []
        for k in list(outputs.keys()):
            if k != 'mesh_files':
                val = self._get_files(subject_path, k,
                                      output_traits.traits()[k].loc,
                                      output_traits.traits()[k].altkey)
                if val:
                    value_list = simplify_list(val)
                    if isinstance(value_list, list):
                        out_files = []
                        for value in value_list:
                            out_files.append(op.abspath(value))
                    elif isinstance(value_list, (str, bytes)):
                        out_files = op.abspath(value_list)
                    else:
                        raise TypeError
                    outputs[k] = out_files
                    if not k.rfind('surface') == -1:
                        mesh_paths.append(out_files)
        outputs['mesh_files'] = mesh_paths
        return outputs



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from nipype.testing import example_data
from nipype.utils import NUMPY_MMAP


def test_split_and_merge(tmpdir):
    import numpy as np
    import nibabel as nb
    import os.path as op
    import os

    from nipype.algorithms.misc import split_rois, merge_rois

    in_mask = example_data('tpms_msk.nii.gz')
    dwfile = tmpdir.join('dwi.nii.gz').strpath
    mskdata = nb.load(in_mask, mmap=NUMPY_MMAP).get_data()
    aff = nb.load(in_mask, mmap=NUMPY_MMAP).affine

    dwshape = (mskdata.shape[0], mskdata.shape[1], mskdata.shape[2], 6)
    dwdata = np.random.normal(size=dwshape)
    tmpdir.chdir()
    nb.Nifti1Image(dwdata.astype(np.float32), aff, None).to_filename(dwfile)

    resdw, resmsk, resid = split_rois(dwfile, in_mask, roishape=(20, 20, 2))
    merged = merge_rois(resdw, resid, in_mask)
    dwmerged = nb.load(merged, mmap=NUMPY_MMAP).get_data()

    dwmasked = dwdata * mskdata[:, :, :, np.newaxis]

    assert np.allclose(dwmasked, dwmerged)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brains import landmarksConstellationAligner


def test_landmarksConstellationAligner_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputLandmarksPaired=dict(
            argstr='--inputLandmarksPaired %s',
            extensions=None,
        ),
        outputLandmarksPaired=dict(
            argstr='--outputLandmarksPaired %s',
            hash_files=False,
        ),
    )
    inputs = landmarksConstellationAligner.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_landmarksConstellationAligner_outputs():
    output_map = dict(outputLandmarksPaired=dict(extensions=None, ), )
    outputs = landmarksConstellationAligner.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import division
from __future__ import unicode_literals
from builtins import open
import os
import numpy as np

from nipype.interfaces.base import Undefined
import nipype.interfaces.fsl.maths as fsl
from nipype.interfaces.fsl import no_fsl

import pytest
from nipype.testing.fixtures import create_files_in_directory_plus_output_type


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_maths_base(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get some fslmaths
    maths = fsl.MathsCommand()

    # Test that we got what we wanted
    assert maths.cmd == "fslmaths"

    # Test that it needs a mandatory argument
    with pytest.raises(ValueError):
        maths.run()

    # Set an in file
    maths.inputs.in_file = "a.nii"
    out_file = "a_maths{}".format(out_ext)

    # Now test the most basic command line
    assert maths.cmdline == "fslmaths a.nii {}".format(
        os.path.join(testdir, out_file))

    # Now test that we can set the various data types
    dtypes = ["float", "char", "int", "short", "double", "input"]
    int_cmdline = "fslmaths -dt {} a.nii " + os.path.join(testdir, out_file)
    out_cmdline = "fslmaths a.nii " + os.path.join(testdir,
                                                   out_file) + " -odt {}"
    duo_cmdline = "fslmaths -dt {} a.nii " + os.path.join(
        testdir, out_file) + " -odt {}"
    for dtype in dtypes:
        foo = fsl.MathsCommand(in_file="a.nii", internal_datatype=dtype)
        assert foo.cmdline == int_cmdline.format(dtype)
        bar = fsl.MathsCommand(in_file="a.nii", output_datatype=dtype)
        assert bar.cmdline == out_cmdline.format(dtype)
        foobar = fsl.MathsCommand(
            in_file="a.nii", internal_datatype=dtype, output_datatype=dtype)
        assert foobar.cmdline == duo_cmdline.format(dtype, dtype)

    # Test that we can ask for an outfile name
    maths.inputs.out_file = "b.nii"
    assert maths.cmdline == "fslmaths a.nii b.nii"


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_changedt(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get some fslmaths
    cdt = fsl.ChangeDataType()

    # Test that we got what we wanted
    assert cdt.cmd == "fslmaths"

    # Test that it needs a mandatory argument
    with pytest.raises(ValueError):
        cdt.run()

    # Set an in file and out file
    cdt.inputs.in_file = "a.nii"
    cdt.inputs.out_file = "b.nii"

    # But it still shouldn't work
    with pytest.raises(ValueError):
        cdt.run()

    # Now test that we can set the various data types
    dtypes = ["float", "char", "int", "short", "double", "input"]
    cmdline = "fslmaths a.nii b.nii -odt {}"
    for dtype in dtypes:
        foo = fsl.MathsCommand(
            in_file="a.nii", out_file="b.nii", output_datatype=dtype)
        assert foo.cmdline == cmdline.format(dtype)


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_threshold(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get the command
    thresh = fsl.Threshold(in_file="a.nii", out_file="b.nii")

    # Test the underlying command
    assert thresh.cmd == "fslmaths"

    # Test mandtory args
    with pytest.raises(ValueError):
        thresh.run()

    # Test the various opstrings
    cmdline = "fslmaths a.nii {} b.nii"
    for val in [0, 0., -1, -1.5, -0.5, 0.5, 3, 400, 400.5]:
        thresh.inputs.thresh = val
        assert thresh.cmdline == cmdline.format("-thr {:.10f}".format(val))

    val = "{:.10f}".format(42)
    thresh = fsl.Threshold(
        in_file="a.nii", out_file="b.nii", thresh=42, use_robust_range=True)
    assert thresh.cmdline == cmdline.format("-thrp " + val)
    thresh.inputs.use_nonzero_voxels = True
    assert thresh.cmdline == cmdline.format("-thrP " + val)
    thresh = fsl.Threshold(
        in_file="a.nii", out_file="b.nii", thresh=42, direction="above")
    assert thresh.cmdline == cmdline.format("-uthr " + val)
    thresh.inputs.use_robust_range = True
    assert thresh.cmdline == cmdline.format("-uthrp " + val)
    thresh.inputs.use_nonzero_voxels = True
    assert thresh.cmdline == cmdline.format("-uthrP " + val)


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_meanimage(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get the command
    meaner = fsl.MeanImage(in_file="a.nii", out_file="b.nii")

    # Test the underlying command
    assert meaner.cmd == "fslmaths"

    # Test the defualt opstring
    assert meaner.cmdline == "fslmaths a.nii -Tmean b.nii"

    # Test the other dimensions
    cmdline = "fslmaths a.nii -{}mean b.nii"
    for dim in ["X", "Y", "Z", "T"]:
        meaner.inputs.dimension = dim
        assert meaner.cmdline == cmdline.format(dim)

    # Test the auto naming
    meaner = fsl.MeanImage(in_file="a.nii")
    assert meaner.cmdline == "fslmaths a.nii -Tmean {}".format(
        os.path.join(testdir, "a_mean{}".format(out_ext)))


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_stdimage(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get the command
    stder = fsl.StdImage(in_file="a.nii", out_file="b.nii")

    # Test the underlying command
    assert stder.cmd == "fslmaths"

    # Test the defualt opstring
    assert stder.cmdline == "fslmaths a.nii -Tstd b.nii"

    # Test the other dimensions
    cmdline = "fslmaths a.nii -{}std b.nii"
    for dim in ["X", "Y", "Z", "T"]:
        stder.inputs.dimension = dim
        assert stder.cmdline == cmdline.format(dim)

    # Test the auto naming
    stder = fsl.StdImage(in_file="a.nii", output_type='NIFTI')
    assert stder.cmdline == "fslmaths a.nii -Tstd {}".format(
        os.path.join(testdir, "a_std.nii"))


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_maximage(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get the command
    maxer = fsl.MaxImage(in_file="a.nii", out_file="b.nii")

    # Test the underlying command
    assert maxer.cmd == "fslmaths"

    # Test the defualt opstring
    assert maxer.cmdline == "fslmaths a.nii -Tmax b.nii"

    # Test the other dimensions
    cmdline = "fslmaths a.nii -{}max b.nii"
    for dim in ["X", "Y", "Z", "T"]:
        maxer.inputs.dimension = dim
        assert maxer.cmdline == cmdline.format(dim)

    # Test the auto naming
    maxer = fsl.MaxImage(in_file="a.nii")
    assert maxer.cmdline == "fslmaths a.nii -Tmax {}".format(
        os.path.join(testdir, "a_max{}".format(out_ext)))


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_smooth(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get the command
    smoother = fsl.IsotropicSmooth(in_file="a.nii", out_file="b.nii")

    # Test the underlying command
    assert smoother.cmd == "fslmaths"

    # Test that smoothing kernel is mandatory
    with pytest.raises(ValueError):
        smoother.run()

    # Test smoothing kernels
    cmdline = "fslmaths a.nii -s {:.5f} b.nii"
    for val in [0, 1., 1, 25, 0.5, 8 / 3.]:
        smoother = fsl.IsotropicSmooth(
            in_file="a.nii", out_file="b.nii", sigma=val)
        assert smoother.cmdline == cmdline.format(val)
        smoother = fsl.IsotropicSmooth(
            in_file="a.nii", out_file="b.nii", fwhm=val)
        val = float(val) / np.sqrt(8 * np.log(2))
        assert smoother.cmdline == cmdline.format(val)

    # Test automatic naming
    smoother = fsl.IsotropicSmooth(in_file="a.nii", sigma=5)
    assert smoother.cmdline == "fslmaths a.nii -s {:.5f} {}".format(
        5, os.path.join(testdir, "a_smooth{}".format(out_ext)))


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_mask(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get the command
    masker = fsl.ApplyMask(in_file="a.nii", out_file="c.nii")

    # Test the underlying command
    assert masker.cmd == "fslmaths"

    # Test that the mask image is mandatory
    with pytest.raises(ValueError):
        masker.run()

    # Test setting the mask image
    masker.inputs.mask_file = "b.nii"
    assert masker.cmdline == "fslmaths a.nii -mas b.nii c.nii"

    # Test auto name generation
    masker = fsl.ApplyMask(in_file="a.nii", mask_file="b.nii")
    assert masker.cmdline == "fslmaths a.nii -mas b.nii " + os.path.join(
        testdir, "a_masked{}".format(out_ext))


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_dilation(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get the command
    diller = fsl.DilateImage(in_file="a.nii", out_file="b.nii")

    # Test the underlying command
    assert diller.cmd == "fslmaths"

    # Test that the dilation operation is mandatory
    with pytest.raises(ValueError):
        diller.run()

    # Test the different dilation operations
    for op in ["mean", "modal", "max"]:
        cv = dict(mean="M", modal="D", max="F")
        diller.inputs.operation = op
        assert diller.cmdline == "fslmaths a.nii -dil{} b.nii".format(cv[op])

    # Now test the different kernel options
    for k in ["3D", "2D", "box", "boxv", "gauss", "sphere"]:
        for size in [1, 1.5, 5]:
            diller.inputs.kernel_shape = k
            diller.inputs.kernel_size = size
            assert diller.cmdline == "fslmaths a.nii -kernel {} {:.4f} -dilF b.nii".format(
                k, size)

    # Test that we can use a file kernel
    f = open("kernel.txt", "w").close()
    del f  # Shut pyflakes up
    diller.inputs.kernel_shape = "file"
    diller.inputs.kernel_size = Undefined
    diller.inputs.kernel_file = "kernel.txt"
    assert diller.cmdline == "fslmaths a.nii -kernel file kernel.txt -dilF b.nii"

    # Test that we don't need to request an out name
    dil = fsl.DilateImage(in_file="a.nii", operation="max")
    assert dil.cmdline == "fslmaths a.nii -dilF {}".format(
        os.path.join(testdir, "a_dil{}".format(out_ext)))


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_erosion(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get the command
    erode = fsl.ErodeImage(in_file="a.nii", out_file="b.nii")

    # Test the underlying command
    assert erode.cmd == "fslmaths"

    # Test the basic command line
    assert erode.cmdline == "fslmaths a.nii -ero b.nii"

    # Test that something else happens when you minimum filter
    erode.inputs.minimum_filter = True
    assert erode.cmdline == "fslmaths a.nii -eroF b.nii"

    # Test that we don't need to request an out name
    erode = fsl.ErodeImage(in_file="a.nii")
    assert erode.cmdline == "fslmaths a.nii -ero {}".format(
        os.path.join(testdir, "a_ero{}".format(out_ext)))


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_spatial_filter(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get the command
    filter = fsl.SpatialFilter(in_file="a.nii", out_file="b.nii")

    # Test the underlying command
    assert filter.cmd == "fslmaths"

    # Test that it fails without an operation
    with pytest.raises(ValueError):
        filter.run()

    # Test the different operations
    for op in ["mean", "meanu", "median"]:
        filter.inputs.operation = op
        assert filter.cmdline == "fslmaths a.nii -f{} b.nii".format(op)

    # Test that we don't need to ask for an out name
    filter = fsl.SpatialFilter(in_file="a.nii", operation="mean")
    assert filter.cmdline == "fslmaths a.nii -fmean {}".format(
        os.path.join(testdir, "a_filt{}".format(out_ext)))


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_unarymaths(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get the command
    maths = fsl.UnaryMaths(in_file="a.nii", out_file="b.nii")

    # Test the underlying command
    assert maths.cmd == "fslmaths"

    # Test that it fails without an operation
    with pytest.raises(ValueError):
        maths.run()

    # Test the different operations
    ops = [
        "exp", "log", "sin", "cos", "sqr", "sqrt", "recip", "abs", "bin",
        "index"
    ]
    for op in ops:
        maths.inputs.operation = op
        assert maths.cmdline == "fslmaths a.nii -{} b.nii".format(op)

    # Test that we don't need to ask for an out file
    for op in ops:
        maths = fsl.UnaryMaths(in_file="a.nii", operation=op)
        assert maths.cmdline == "fslmaths a.nii -{} {}".format(
            op, os.path.join(testdir, "a_{}{}".format(op, out_ext)))


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_binarymaths(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get the command
    maths = fsl.BinaryMaths(in_file="a.nii", out_file="c.nii")

    # Test the underlying command
    assert maths.cmd == "fslmaths"

    # Test that it fails without an operation an
    with pytest.raises(ValueError):
        maths.run()

    # Test the different operations
    ops = ["add", "sub", "mul", "div", "rem", "min", "max"]
    operands = ["b.nii", -2, -0.5, 0, .123456, np.pi, 500]
    for op in ops:
        for ent in operands:
            maths = fsl.BinaryMaths(
                in_file="a.nii", out_file="c.nii", operation=op)
            if ent == "b.nii":
                maths.inputs.operand_file = ent
                assert maths.cmdline == "fslmaths a.nii -{} b.nii c.nii".format(
                    op)
            else:
                maths.inputs.operand_value = ent
                assert maths.cmdline == "fslmaths a.nii -{} {:.8f} c.nii".format(
                    op, ent)

    # Test that we don't need to ask for an out file
    for op in ops:
        maths = fsl.BinaryMaths(
            in_file="a.nii", operation=op, operand_file="b.nii")
        assert maths.cmdline == "fslmaths a.nii -{} b.nii {}".format(
            op, os.path.join(testdir, "a_maths{}".format(out_ext)))


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_multimaths(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get the command
    maths = fsl.MultiImageMaths(in_file="a.nii", out_file="c.nii")

    # Test the underlying command
    assert maths.cmd == "fslmaths"

    # Test that it fails without an operation an
    with pytest.raises(ValueError):
        maths.run()

    # Test a few operations
    maths.inputs.operand_files = ["a.nii", "b.nii"]
    opstrings = [
        "-add %s -div %s", "-max 1 -sub %s -min %s", "-mas %s -add %s"
    ]
    for ostr in opstrings:
        maths.inputs.op_string = ostr
        assert maths.cmdline == "fslmaths a.nii %s c.nii" % ostr % ("a.nii",
                                                                    "b.nii")

    # Test that we don't need to ask for an out file
    maths = fsl.MultiImageMaths(
        in_file="a.nii", op_string="-add %s -mul 5", operand_files=["b.nii"])
    assert maths.cmdline == \
        "fslmaths a.nii -add b.nii -mul 5 %s" % os.path.join(testdir, "a_maths%s" % out_ext)


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_tempfilt(create_files_in_directory_plus_output_type):
    files, testdir, out_ext = create_files_in_directory_plus_output_type

    # Get the command
    filt = fsl.TemporalFilter(in_file="a.nii", out_file="b.nii")

    # Test the underlying command
    assert filt.cmd == "fslmaths"

    # Test that both filters are initialized off
    assert filt.cmdline == "fslmaths a.nii -bptf -1.000000 -1.000000 b.nii"

    # Test some filters
    windows = [(-1, -1), (0.1, 0.1), (-1, 20), (20, -1), (128, 248)]
    for win in windows:
        filt.inputs.highpass_sigma = win[0]
        filt.inputs.lowpass_sigma = win[1]
        assert filt.cmdline == "fslmaths a.nii -bptf {:.6f} {:.6f} b.nii".format(
            win[0], win[1])

    # Test that we don't need to ask for an out file
    filt = fsl.TemporalFilter(in_file="a.nii", highpass_sigma=64)
    assert filt.cmdline == \
        "fslmaths a.nii -bptf 64.000000 -1.000000 {}".format(os.path.join(testdir, "a_filt{}".format(out_ext)))



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

import pytest

from ....testing import example_data
from ...niftyreg import get_custom_path
from ...niftyreg.tests.test_regutils import no_nifty_tool
from .. import UnaryStats, BinaryStats


@pytest.mark.skipif(
    no_nifty_tool(cmd='seg_stats'), reason="niftyseg is not installed")
def test_unary_stats():
    """ Test for the seg_stats interfaces """
    # Create a node object
    unarys = UnaryStats()

    # Check if the command is properly defined
    cmd = get_custom_path('seg_stats', env_dir='NIFTYSEGDIR')
    assert unarys.cmd == cmd

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        unarys.run()

    # Assign some input data
    in_file = example_data('im1.nii')
    unarys.inputs.in_file = in_file
    unarys.inputs.operation = 'a'

    expected_cmd = '{cmd} {in_file} -a'.format(cmd=cmd, in_file=in_file)

    assert unarys.cmdline == expected_cmd


@pytest.mark.skipif(
    no_nifty_tool(cmd='seg_stats'), reason="niftyseg is not installed")
def test_binary_stats():
    """ Test for the seg_stats interfaces """
    # Create a node object
    binarys = BinaryStats()

    # Check if the command is properly defined
    cmd = get_custom_path('seg_stats', env_dir='NIFTYSEGDIR')
    assert binarys.cmd == cmd

    # test raising error with mandatory args absent
    with pytest.raises(ValueError):
        binarys.run()

    # Assign some input data
    in_file = example_data('im1.nii')
    binarys.inputs.in_file = in_file
    binarys.inputs.operand_value = 2
    binarys.inputs.operation = 'sa'

    expected_cmd = '{cmd} {in_file} -sa 2.00000000'.format(
        cmd=cmd, in_file=in_file)

    assert binarys.cmdline == expected_cmd



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import PRELUDE


def test_PRELUDE_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        complex_phase_file=dict(
            argstr='--complex=%s',
            extensions=None,
            mandatory=True,
            xor=['magnitude_file', 'phase_file'],
        ),
        end=dict(argstr='--end=%d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        label_file=dict(
            argstr='--labels=%s',
            extensions=None,
            hash_files=False,
        ),
        labelprocess2d=dict(argstr='--labelslices', ),
        magnitude_file=dict(
            argstr='--abs=%s',
            extensions=None,
            mandatory=True,
            xor=['complex_phase_file'],
        ),
        mask_file=dict(
            argstr='--mask=%s',
            extensions=None,
        ),
        num_partitions=dict(argstr='--numphasesplit=%d', ),
        output_type=dict(),
        phase_file=dict(
            argstr='--phase=%s',
            extensions=None,
            mandatory=True,
            xor=['complex_phase_file'],
        ),
        process2d=dict(
            argstr='--slices',
            xor=['labelprocess2d'],
        ),
        process3d=dict(
            argstr='--force3D',
            xor=['labelprocess2d', 'process2d'],
        ),
        rawphase_file=dict(
            argstr='--rawphase=%s',
            extensions=None,
            hash_files=False,
        ),
        removeramps=dict(argstr='--removeramps', ),
        savemask_file=dict(
            argstr='--savemask=%s',
            extensions=None,
            hash_files=False,
        ),
        start=dict(argstr='--start=%d', ),
        threshold=dict(argstr='--thresh=%.10f', ),
        unwrapped_phase_file=dict(
            argstr='--unwrap=%s',
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
    )
    inputs = PRELUDE.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_PRELUDE_outputs():
    output_map = dict(unwrapped_phase_file=dict(extensions=None, ), )
    outputs = PRELUDE.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import ReconAll


def test_ReconAll_inputs():
    input_map = dict(
        FLAIR_file=dict(
            argstr='-FLAIR %s',
            extensions=None,
            min_ver='5.3.0',
        ),
        T1_files=dict(argstr='-i %s...', ),
        T2_file=dict(
            argstr='-T2 %s',
            extensions=None,
            min_ver='5.3.0',
        ),
        args=dict(argstr='%s', ),
        big_ventricles=dict(argstr='-bigventricles', ),
        brainstem=dict(argstr='-brainstem-structures', ),
        directive=dict(
            argstr='-%s',
            position=0,
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        expert=dict(
            argstr='-expert %s',
            extensions=None,
        ),
        flags=dict(argstr='%s', ),
        hemi=dict(argstr='-hemi %s', ),
        hippocampal_subfields_T1=dict(
            argstr='-hippocampal-subfields-T1',
            min_ver='6.0.0',
        ),
        hippocampal_subfields_T2=dict(
            argstr='-hippocampal-subfields-T2 %s %s',
            min_ver='6.0.0',
        ),
        hires=dict(
            argstr='-hires',
            min_ver='6.0.0',
        ),
        mprage=dict(argstr='-mprage', ),
        mri_aparc2aseg=dict(xor=['expert'], ),
        mri_ca_label=dict(xor=['expert'], ),
        mri_ca_normalize=dict(xor=['expert'], ),
        mri_ca_register=dict(xor=['expert'], ),
        mri_edit_wm_with_aseg=dict(xor=['expert'], ),
        mri_em_register=dict(xor=['expert'], ),
        mri_fill=dict(xor=['expert'], ),
        mri_mask=dict(xor=['expert'], ),
        mri_normalize=dict(xor=['expert'], ),
        mri_pretess=dict(xor=['expert'], ),
        mri_remove_neck=dict(xor=['expert'], ),
        mri_segment=dict(xor=['expert'], ),
        mri_segstats=dict(xor=['expert'], ),
        mri_tessellate=dict(xor=['expert'], ),
        mri_watershed=dict(xor=['expert'], ),
        mris_anatomical_stats=dict(xor=['expert'], ),
        mris_ca_label=dict(xor=['expert'], ),
        mris_fix_topology=dict(xor=['expert'], ),
        mris_inflate=dict(xor=['expert'], ),
        mris_make_surfaces=dict(xor=['expert'], ),
        mris_register=dict(xor=['expert'], ),
        mris_smooth=dict(xor=['expert'], ),
        mris_sphere=dict(xor=['expert'], ),
        mris_surf2vol=dict(xor=['expert'], ),
        mrisp_paint=dict(xor=['expert'], ),
        openmp=dict(argstr='-openmp %d', ),
        parallel=dict(argstr='-parallel', ),
        subject_id=dict(
            argstr='-subjid %s',
            usedefault=True,
        ),
        subjects_dir=dict(
            argstr='-sd %s',
            genfile=True,
            hash_files=False,
        ),
        talairach=dict(xor=['expert'], ),
        use_FLAIR=dict(
            argstr='-FLAIRpial',
            min_ver='5.3.0',
            xor=['use_T2'],
        ),
        use_T2=dict(
            argstr='-T2pial',
            min_ver='5.3.0',
            xor=['use_FLAIR'],
        ),
        xopts=dict(argstr='-xopts-%s', ),
    )
    inputs = ReconAll.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ReconAll_outputs():
    output_map = dict(
        BA_stats=dict(
            altkey='BA',
            loc='stats',
        ),
        T1=dict(
            extensions=None,
            loc='mri',
        ),
        annot=dict(
            altkey='*annot',
            loc='label',
        ),
        aparc_a2009s_stats=dict(
            altkey='aparc.a2009s',
            loc='stats',
        ),
        aparc_aseg=dict(
            altkey='aparc*aseg',
            loc='mri',
        ),
        aparc_stats=dict(
            altkey='aparc',
            loc='stats',
        ),
        area_pial=dict(
            altkey='area.pial',
            loc='surf',
        ),
        aseg=dict(
            extensions=None,
            loc='mri',
        ),
        aseg_stats=dict(
            altkey='aseg',
            loc='stats',
        ),
        avg_curv=dict(loc='surf', ),
        brain=dict(
            extensions=None,
            loc='mri',
        ),
        brainmask=dict(
            extensions=None,
            loc='mri',
        ),
        curv=dict(loc='surf', ),
        curv_pial=dict(
            altkey='curv.pial',
            loc='surf',
        ),
        curv_stats=dict(
            altkey='curv',
            loc='stats',
        ),
        entorhinal_exvivo_stats=dict(
            altkey='entorhinal_exvivo',
            loc='stats',
        ),
        filled=dict(
            extensions=None,
            loc='mri',
        ),
        graymid=dict(
            altkey=['graymid', 'midthickness'],
            loc='surf',
        ),
        inflated=dict(loc='surf', ),
        jacobian_white=dict(loc='surf', ),
        label=dict(
            altkey='*label',
            loc='label',
        ),
        norm=dict(
            extensions=None,
            loc='mri',
        ),
        nu=dict(
            extensions=None,
            loc='mri',
        ),
        orig=dict(
            extensions=None,
            loc='mri',
        ),
        pial=dict(loc='surf', ),
        rawavg=dict(
            extensions=None,
            loc='mri',
        ),
        ribbon=dict(
            altkey='*ribbon',
            loc='mri',
        ),
        smoothwm=dict(loc='surf', ),
        sphere=dict(loc='surf', ),
        sphere_reg=dict(
            altkey='sphere.reg',
            loc='surf',
        ),
        subject_id=dict(),
        subjects_dir=dict(),
        sulc=dict(loc='surf', ),
        thickness=dict(loc='surf', ),
        volume=dict(loc='surf', ),
        white=dict(loc='surf', ),
        wm=dict(
            extensions=None,
            loc='mri',
        ),
        wmparc=dict(
            extensions=None,
            loc='mri',
        ),
        wmparc_stats=dict(
            altkey='wmparc',
            loc='stats',
        ),
    )
    outputs = ReconAll.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# coding: utf-8
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Interfaces to perform image registrations and to apply the resulting
displacement maps to images and points.

"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import open

import os.path as op
import re

from ... import logging
from .base import ElastixBaseInputSpec
from ..base import CommandLine, TraitedSpec, File, traits, InputMultiPath

iflogger = logging.getLogger('nipype.interface')


class RegistrationInputSpec(ElastixBaseInputSpec):
    fixed_image = File(
        exists=True, mandatory=True, argstr='-f %s', desc='fixed image')
    moving_image = File(
        exists=True, mandatory=True, argstr='-m %s', desc='moving image')
    parameters = InputMultiPath(
        File(exists=True),
        mandatory=True,
        argstr='-p %s...',
        desc='parameter file, elastix handles 1 or more -p')
    fixed_mask = File(
        exists=True, argstr='-fMask %s', desc='mask for fixed image')
    moving_mask = File(
        exists=True, argstr='-mMask %s', desc='mask for moving image')
    initial_transform = File(
        exists=True,
        argstr='-t0 %s',
        desc='parameter file for initial transform')


class RegistrationOutputSpec(TraitedSpec):
    transform = InputMultiPath(File(exists=True), desc='output transform')
    warped_file = File(desc='input moving image warped to fixed image')
    warped_files = InputMultiPath(
        File(exists=False),
        desc=('input moving image warped to fixed image at each level'))
    warped_files_flags = traits.List(
        traits.Bool(False),
        desc='flag indicating if warped image was generated')


class Registration(CommandLine):
    """
    Elastix nonlinear registration interface

    Example
    -------

    >>> from nipype.interfaces.elastix import Registration
    >>> reg = Registration()
    >>> reg.inputs.fixed_image = 'fixed1.nii'
    >>> reg.inputs.moving_image = 'moving1.nii'
    >>> reg.inputs.parameters = ['elastix.txt']
    >>> reg.cmdline
    'elastix -f fixed1.nii -m moving1.nii -threads 1 -out ./ -p elastix.txt'


    """

    _cmd = 'elastix'
    input_spec = RegistrationInputSpec
    output_spec = RegistrationOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()

        out_dir = op.abspath(self.inputs.output_path)

        regex = re.compile(r'^\((\w+)\s(.+)\)$')

        outputs['transform'] = []
        outputs['warped_files'] = []
        outputs['warped_files_flags'] = []

        for i, params in enumerate(self.inputs.parameters):
            config = {}

            with open(params, 'r') as f:
                for line in f.readlines():
                    line = line.strip()
                    if not line.startswith('//') and line:
                        m = regex.search(line)
                        if m:
                            value = self._cast(m.group(2).strip())
                            config[m.group(1).strip()] = value

            outputs['transform'].append(
                op.join(out_dir, 'TransformParameters.%01d.txt' % i))

            warped_file = None
            if config['WriteResultImage']:
                warped_file = op.join(out_dir, 'result.%01d.%s' %
                                      (i, config['ResultImageFormat']))

            outputs['warped_files'].append(warped_file)
            outputs['warped_files_flags'].append(config['WriteResultImage'])

        if outputs['warped_files_flags'][-1]:
            outputs['warped_file'] = outputs['warped_files'][-1]

        return outputs

    def _cast(self, val):
        if val.startswith('"') and val.endswith('"'):
            if val == '"true"':
                return True
            elif val == '"false"':
                return False
            else:
                return val[1:-1]

        try:
            return int(val)
        except ValueError:
            try:
                return float(val)
            except ValueError:
                return val


class ApplyWarpInputSpec(ElastixBaseInputSpec):
    transform_file = File(
        exists=True,
        mandatory=True,
        argstr='-tp %s',
        desc='transform-parameter file, only 1')

    moving_image = File(
        exists=True,
        argstr='-in %s',
        mandatory=True,
        desc='input image to deform')


class ApplyWarpOutputSpec(TraitedSpec):
    warped_file = File(desc='input moving image warped to fixed image')


class ApplyWarp(CommandLine):
    """
    Use ``transformix`` to apply a transform on an input image.
    The transform is specified in the transform-parameter file.

    Example
    -------

    >>> from nipype.interfaces.elastix import ApplyWarp
    >>> reg = ApplyWarp()
    >>> reg.inputs.moving_image = 'moving1.nii'
    >>> reg.inputs.transform_file = 'TransformParameters.0.txt'
    >>> reg.cmdline
    'transformix -in moving1.nii -threads 1 -out ./ -tp TransformParameters.0.txt'


    """

    _cmd = 'transformix'
    input_spec = ApplyWarpInputSpec
    output_spec = ApplyWarpOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()
        out_dir = op.abspath(self.inputs.output_path)
        outputs['warped_file'] = op.join(out_dir, 'result.nii.gz')
        return outputs


class AnalyzeWarpInputSpec(ApplyWarpInputSpec):
    points = traits.Enum(
        'all',
        usedefault=True,
        position=0,
        argstr='-def %s',
        desc='transform all points from the input-image, which effectively'
             ' generates a deformation field.')
    jac = traits.Enum(
        'all',
        usedefault=True,
        argstr='-jac %s',
        desc='generate an image with the determinant of the spatial Jacobian')
    jacmat = traits.Enum(
        'all',
        usedefault=True,
        argstr='-jacmat %s',
        desc='generate an image with the spatial Jacobian matrix at each voxel')
    moving_image = File(
        exists=True,
        argstr='-in %s',
        desc='input image to deform (not used)')

class AnalyzeWarpOutputSpec(TraitedSpec):
    disp_field = File(desc='displacements field')
    jacdet_map = File(desc='det(Jacobian) map')
    jacmat_map = File(desc='Jacobian matrix map')


class AnalyzeWarp(ApplyWarp):
    """
    Use transformix to get details from the input transform (generate
    the corresponding deformation field, generate the determinant of the
    Jacobian map or the Jacobian map itself)

    Example
    -------

    >>> from nipype.interfaces.elastix import AnalyzeWarp
    >>> reg = AnalyzeWarp()
    >>> reg.inputs.transform_file = 'TransformParameters.0.txt'
    >>> reg.cmdline
    'transformix -def all -jac all -jacmat all -threads 1 -out ./ -tp TransformParameters.0.txt'


    """

    input_spec = AnalyzeWarpInputSpec
    output_spec = AnalyzeWarpOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()
        out_dir = op.abspath(self.inputs.output_path)
        outputs['disp_field'] = op.join(out_dir, 'deformationField.nii.gz')
        outputs['jacdet_map'] = op.join(out_dir, 'spatialJacobian.nii.gz')
        outputs['jacmat_map'] = op.join(out_dir, 'fullSpatialJacobian.nii.gz')
        return outputs


class PointsWarpInputSpec(ElastixBaseInputSpec):
    points_file = File(
        exists=True,
        argstr='-def %s',
        mandatory=True,
        desc='input points (accepts .vtk triangular meshes).')
    transform_file = File(
        exists=True,
        mandatory=True,
        argstr='-tp %s',
        desc='transform-parameter file, only 1')


class PointsWarpOutputSpec(TraitedSpec):
    warped_file = File(desc='input points displaced in fixed image domain')


class PointsWarp(CommandLine):
    """Use ``transformix`` to apply a transform on an input point set.
    The transform is specified in the transform-parameter file.

    Example
    -------

    >>> from nipype.interfaces.elastix import PointsWarp
    >>> reg = PointsWarp()
    >>> reg.inputs.points_file = 'surf1.vtk'
    >>> reg.inputs.transform_file = 'TransformParameters.0.txt'
    >>> reg.cmdline
    'transformix -threads 1 -out ./ -def surf1.vtk -tp TransformParameters.0.txt'


    """

    _cmd = 'transformix'
    input_spec = PointsWarpInputSpec
    output_spec = PointsWarpOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()
        out_dir = op.abspath(self.inputs.output_path)

        fname, ext = op.splitext(op.basename(self.inputs.points_file))

        outputs['warped_file'] = op.join(out_dir, 'outputpoints%s' % ext)
        return outputs



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Miscellaneous utility functions
"""
from __future__ import (print_function, unicode_literals, division,
                        absolute_import)
from builtins import next, bytes, str

import os
import sys
import re
from collections import Iterator
from warnings import warn

from distutils.version import LooseVersion

import numpy as np
from future.utils import raise_from
from future import standard_library
try:
    from textwrap import indent as textwrap_indent
except ImportError:

    def textwrap_indent(text, prefix):
        """ A textwrap.indent replacement for Python < 3.3 """
        if not prefix:
            return text
        splittext = text.splitlines(True)
        return prefix + prefix.join(splittext)


standard_library.install_aliases()


def human_order_sorted(l):
    """Sorts string in human order (i.e. 'stat10' will go after 'stat2')"""

    def atoi(text):
        return int(text) if text.isdigit() else text

    def natural_keys(text):
        if isinstance(text, tuple):
            text = text[0]
        return [atoi(c) for c in re.split('(\d+)', text)]

    return sorted(l, key=natural_keys)


def trim(docstring, marker=None):
    if isinstance(docstring, bytes):
        docstring = str(docstring, 'utf-8')

    if not docstring:
        return ''
    # Convert tabs to spaces (following the normal Python rules)
    # and split into a list of lines:
    lines = docstring.expandtabs().splitlines()
    # Determine minimum indentation (first line doesn't count):
    indent = sys.maxsize
    for line in lines[1:]:
        stripped = line.lstrip()
        if stripped:
            indent = min(indent, len(line) - len(stripped))
    # Remove indentation (first line is special):
    trimmed = [lines[0].strip()]
    if indent < sys.maxsize:
        for line in lines[1:]:
            # replace existing REST marker with doc level marker
            stripped = line.lstrip().strip().rstrip()
            if marker is not None and stripped and \
               all([s == stripped[0] for s in stripped]) and \
               stripped[0] not in [':']:
                line = line.replace(stripped[0], marker)
            trimmed.append(line[indent:].rstrip())
    # Strip off trailing and leading blank lines:
    while trimmed and not trimmed[-1]:
        trimmed.pop()
    while trimmed and not trimmed[0]:
        trimmed.pop(0)
    # Return a single string:
    return '\n'.join(trimmed)


def find_indices(condition):
    "Return the indices where ravel(condition) is true"
    res, = np.nonzero(np.ravel(condition))
    return res


def is_container(item):
    """Checks if item is a container (list, tuple, dict, set)

    Parameters
    ----------
    item : object
        object to check for .__iter__

    Returns
    -------
    output : Boolean
        True if container
        False if not (eg string)
    """
    if isinstance(item, str):
        return False
    elif hasattr(item, '__iter__'):
        return True
    else:
        return False


def container_to_string(cont):
    """Convert a container to a command line string.

    Elements of the container are joined with a space between them,
    suitable for a command line parameter.

    If the container `cont` is only a sequence, like a string and not a
    container, it is returned unmodified.

    Parameters
    ----------
    cont : container
       A container object like a list, tuple, dict, or a set.

    Returns
    -------
    cont_str : string
        Container elements joined into a string.

    """
    if hasattr(cont, '__iter__') and not isinstance(cont, str):
        cont = ' '.join(cont)
    return str(cont)


# Dependency checks.  Copied this from Nipy, with some modificiations
# (added app as a parameter).
def package_check(pkg_name,
                  version=None,
                  app=None,
                  checker=LooseVersion,
                  exc_failed_import=ImportError,
                  exc_failed_check=RuntimeError):
    """Check that the minimal version of the required package is installed.

    Parameters
    ----------
    pkg_name : string
        Name of the required package.
    version : string, optional
        Minimal version number for required package.
    app : string, optional
        Application that is performing the check.  For instance, the
        name of the tutorial being executed that depends on specific
        packages.  Default is *Nipype*.
    checker : object, optional
        The class that will perform the version checking.  Default is
        distutils.version.LooseVersion.
    exc_failed_import : Exception, optional
        Class of the exception to be thrown if import failed.
    exc_failed_check : Exception, optional
        Class of the exception to be thrown if version check failed.

    Examples
    --------
    package_check('numpy', '1.3')
    package_check('scipy', '0.7', 'tutorial1')

    """

    if app:
        msg = '%s requires %s' % (app, pkg_name)
    else:
        msg = 'Nipype requires %s' % pkg_name
    if version:
        msg += ' with version >= %s' % (version, )
    try:
        mod = __import__(pkg_name)
    except ImportError as e:
        raise_from(exc_failed_import(msg), e)
    if not version:
        return
    try:
        have_version = mod.__version__
    except AttributeError as e:
        raise_from(
            exc_failed_check('Cannot find version for %s' % pkg_name), e)
    if checker(have_version) < checker(version):
        raise exc_failed_check(msg)


def str2bool(v):
    """
    Convert strings (and bytearrays) to boolean values

    >>> all([str2bool(v) for v in (True, "yes", "true",
    ...      "y", "t", "Yes", "True", "1", "on", "On")])
    True
    >>> all([str2bool(v.encode('utf-8'))
    ...      for v in ("yes", "true", "y", "t", "1", "Yes", "on", "On")])
    True
    >>> any([str2bool(v) for v in (False, "no", "false", "n", "f",
    ...      "False", "0", "off", "Off")])
    False
    >>> any([str2bool(v.encode('utf-8'))
    ...      for v in ("no", "false", "n", "f", "0", "off", "Off")])
    False
    >>> str2bool(None)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    ValueError: ...
    >>> str2bool('/some/path')  # doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    ValueError: ...
    >>> str2bool('Agg')  # doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    ValueError: ...
    >>> str2bool('INFO')  # doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    ValueError: ...
    >>> str2bool('/some/bytes/path'.encode('utf-8'))  # doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    ValueError: ...

    """
    if isinstance(v, bool):
        return v

    if isinstance(v, bytes):
        v = v.decode('utf-8')

    if isinstance(v, str):
        lower = v.lower()
        if lower in ("yes", "true", "y", "t", "1", "on"):
            return True
        elif lower in ("no", "false", "n", "f", "0", "off"):
            return False

    raise ValueError("%r cannot be converted to bool" % v)


def flatten(S):
    if S == []:
        return S
    if isinstance(S[0], list):
        return flatten(S[0]) + flatten(S[1:])
    return S[:1] + flatten(S[1:])


def unflatten(in_list, prev_structure):
    if not isinstance(in_list, Iterator):
        in_list = iter(in_list)

    if not isinstance(prev_structure, list):
        return next(in_list)

    out = []
    for item in prev_structure:
        out.append(unflatten(in_list, item))
    return out


def normalize_mc_params(params, source):
    """
    Normalize a single row of motion parameters to the SPM format.

    SPM saves motion parameters as:
        x   Right-Left          (mm)
        y   Anterior-Posterior  (mm)
        z   Superior-Inferior   (mm)
        rx  Pitch               (rad)
        ry  Roll                (rad)
        rz  Yaw                 (rad)
    """
    if source.upper() == 'FSL':
        params = params[[3, 4, 5, 0, 1, 2]]
    elif source.upper() in ('AFNI', 'FSFAST'):
        params = params[np.asarray([4, 5, 3, 1, 2, 0]) + (len(params) > 6)]
        params[3:] = params[3:] * np.pi / 180.
    elif source.upper() == 'NIPY':
        from nipy.algorithms.registration import to_matrix44, aff2euler
        matrix = to_matrix44(params)
        params = np.zeros(6)
        params[:3] = matrix[:3, 3]
        params[-1:2:-1] = aff2euler(matrix)

    return params


def dict_diff(dold, dnew, indent=0):
    """Helper to log what actually changed from old to new values of
    dictionaries.

    typical use -- log difference for hashed_inputs
    """
    # First check inputs, since they usually are lists of tuples
    # and dicts are required.
    if isinstance(dnew, list):
        dnew = dict(dnew)
    if isinstance(dold, list):
        dold = dict(dold)

    # Compare against hashed_inputs
    # Keys: should rarely differ
    new_keys = set(dnew.keys())
    old_keys = set(dold.keys())

    diff = []
    if new_keys - old_keys:
        diff += ["  * keys not previously seen: %s" % (new_keys - old_keys)]

    if old_keys - new_keys:
        diff += ["  * keys not presently seen: %s" % (old_keys - new_keys)]

    # Add topical message
    if diff:
        diff.insert(0, "Dictionaries had differing keys:")

    diffkeys = len(diff)

    # Values in common keys would differ quite often,
    # so we need to join the messages together
    for k in new_keys.intersection(old_keys):
        try:
            new, old = dnew[k], dold[k]
            same = new == old
            if not same:
                # Since JSON does not discriminate between lists and
                # tuples, we might need to cast them into the same type
                # as the last resort.  And lets try to be more generic
                same = old.__class__(new) == old
        except Exception:
            same = False
        if not same:
            diff += ["  * %s: %r != %r" % (k, dnew[k], dold[k])]

    if len(diff) > diffkeys:
        diff.insert(diffkeys, "Some dictionary entries had differing values:")

    return textwrap_indent('\n'.join(diff), ' ' * indent)


def rgetcwd(error=True):
    """
    Robust replacement for getcwd when folders get removed
    If error==True, this is just an alias for os.getcwd()
    """
    if error:
        return os.getcwd()

    try:
        cwd = os.getcwd()
    except OSError as exc:
        # Changing back to cwd is probably not necessary
        # but this makes sure there's somewhere to change to.
        cwd = os.getenv('PWD')
        if cwd is None:
            raise OSError((
                exc.errno, 'Current directory does not exist anymore, '
                'and nipype was not able to guess it from the environment'))
        warn('Current folder does not exist, replacing with "%s" instead.' % cwd)
    return cwd



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import Slicer


def test_Slicer_inputs():
    input_map = dict(
        all_axial=dict(
            argstr='-A',
            position=10,
            requires=['image_width'],
            xor=('single_slice', 'middle_slices', 'all_axial', 'sample_axial'),
        ),
        args=dict(argstr='%s', ),
        colour_map=dict(
            argstr='-l %s',
            extensions=None,
            position=4,
        ),
        dither_edges=dict(
            argstr='-t',
            position=7,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        image_edges=dict(
            argstr='%s',
            extensions=None,
            position=2,
        ),
        image_width=dict(
            argstr='%d',
            position=-2,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        intensity_range=dict(
            argstr='-i %.3f %.3f',
            position=5,
        ),
        label_slices=dict(
            argstr='-L',
            position=3,
            usedefault=True,
        ),
        middle_slices=dict(
            argstr='-a',
            position=10,
            xor=('single_slice', 'middle_slices', 'all_axial', 'sample_axial'),
        ),
        nearest_neighbour=dict(
            argstr='-n',
            position=8,
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            hash_files=False,
            position=-1,
        ),
        output_type=dict(),
        sample_axial=dict(
            argstr='-S %d',
            position=10,
            requires=['image_width'],
            xor=('single_slice', 'middle_slices', 'all_axial', 'sample_axial'),
        ),
        scaling=dict(
            argstr='-s %f',
            position=0,
        ),
        show_orientation=dict(
            argstr='%s',
            position=9,
            usedefault=True,
        ),
        single_slice=dict(
            argstr='-%s',
            position=10,
            requires=['slice_number'],
            xor=('single_slice', 'middle_slices', 'all_axial', 'sample_axial'),
        ),
        slice_number=dict(
            argstr='-%d',
            position=11,
        ),
        threshold_edges=dict(
            argstr='-e %.3f',
            position=6,
        ),
    )
    inputs = Slicer.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Slicer_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = Slicer.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

from io import open

import pytest
from nipype.testing import example_data
from nipype.algorithms.confounds import FramewiseDisplacement, ComputeDVARS, \
    is_outlier
import numpy as np

nonitime = True
try:
    import nitime
    nonitime = False
except ImportError:
    pass


def test_fd(tmpdir):
    tempdir = tmpdir.strpath
    ground_truth = np.loadtxt(example_data('fsl_motion_outliers_fd.txt'))
    fdisplacement = FramewiseDisplacement(
        in_file=example_data('fsl_mcflirt_movpar.txt'),
        out_file=tempdir + '/fd.txt',
        parameter_source="FSL")
    res = fdisplacement.run()

    with open(res.outputs.out_file) as all_lines:
        for line in all_lines:
            assert 'FramewiseDisplacement' in line
            break

    assert np.allclose(
        ground_truth, np.loadtxt(res.outputs.out_file, skiprows=1), atol=.16)
    assert np.abs(ground_truth.mean() - res.outputs.fd_average) < 1e-2


@pytest.mark.skipif(nonitime, reason="nitime is not installed")
def test_dvars(tmpdir):
    ground_truth = np.loadtxt(example_data('ds003_sub-01_mc.DVARS'))
    dvars = ComputeDVARS(
        in_file=example_data('ds003_sub-01_mc.nii.gz'),
        in_mask=example_data('ds003_sub-01_mc_brainmask.nii.gz'),
        save_all=True,
        intensity_normalization=0)
    tmpdir.chdir()
    res = dvars.run()

    dv1 = np.loadtxt(res.outputs.out_all, skiprows=1)
    assert (np.abs(dv1[:, 0] - ground_truth[:, 0]).sum() / len(dv1)) < 0.05

    assert (np.abs(dv1[:, 1] - ground_truth[:, 1]).sum() / len(dv1)) < 0.05

    assert (np.abs(dv1[:, 2] - ground_truth[:, 2]).sum() / len(dv1)) < 0.05

    dvars = ComputeDVARS(
        in_file=example_data('ds003_sub-01_mc.nii.gz'),
        in_mask=example_data('ds003_sub-01_mc_brainmask.nii.gz'),
        save_all=True)
    res = dvars.run()

    dv1 = np.loadtxt(res.outputs.out_all, skiprows=1)
    assert (np.abs(dv1[:, 0] - ground_truth[:, 0]).sum() / len(dv1)) < 0.05

    assert (np.abs(dv1[:, 1] - ground_truth[:, 1]).sum() / len(dv1)) > 0.05

    assert (np.abs(dv1[:, 2] - ground_truth[:, 2]).sum() / len(dv1)) < 0.05


def test_outliers():
    np.random.seed(0)
    in_data = np.random.randn(100)
    in_data[0] += 10

    assert is_outlier(in_data) == 1



# -*- coding: utf-8 -*-
from nipype.interfaces.fsl.model import FILMGLS, FILMGLSInputSpec


def test_filmgls():
    input_map = dict(
        args=dict(argstr='%s', ),
        autocorr_estimate_only=dict(
            xor=[
                'autocorr_estimate_only', 'fit_armodel', 'tukey_window',
                'multitaper_product', 'use_pava', 'autocorr_noestimate'
            ],
            argstr='-ac',
        ),
        autocorr_noestimate=dict(
            xor=[
                'autocorr_estimate_only', 'fit_armodel', 'tukey_window',
                'multitaper_product', 'use_pava', 'autocorr_noestimate'
            ],
            argstr='-noest',
        ),
        brightness_threshold=dict(argstr='-epith %d', ),
        design_file=dict(argstr='%s', ),
        environ=dict(usedefault=True, ),
        fit_armodel=dict(
            xor=[
                'autocorr_estimate_only', 'fit_armodel', 'tukey_window',
                'multitaper_product', 'use_pava', 'autocorr_noestimate'
            ],
            argstr='-ar',
        ),
        full_data=dict(argstr='-v', ),
        in_file=dict(
            mandatory=True,
            argstr='%s',
        ),
        mask_size=dict(argstr='-ms %d', ),
        multitaper_product=dict(
            xor=[
                'autocorr_estimate_only', 'fit_armodel', 'tukey_window',
                'multitaper_product', 'use_pava', 'autocorr_noestimate'
            ],
            argstr='-mt %d',
        ),
        output_pwdata=dict(argstr='-output_pwdata', ),
        output_type=dict(),
        results_dir=dict(
            usedefault=True,
            argstr='-rn %s',
        ),
        smooth_autocorr=dict(argstr='-sa', ),
        threshold=dict(argstr='%f', ),
        tukey_window=dict(
            xor=[
                'autocorr_estimate_only', 'fit_armodel', 'tukey_window',
                'multitaper_product', 'use_pava', 'autocorr_noestimate'
            ],
            argstr='-tukey %d',
        ),
        use_pava=dict(argstr='-pava', ),
    )
    input_map2 = dict(
        args=dict(argstr='%s', ),
        autocorr_estimate_only=dict(
            xor=[
                'autocorr_estimate_only', 'fit_armodel', 'tukey_window',
                'multitaper_product', 'use_pava', 'autocorr_noestimate'
            ],
            argstr='--ac',
        ),
        autocorr_noestimate=dict(
            xor=[
                'autocorr_estimate_only', 'fit_armodel', 'tukey_window',
                'multitaper_product', 'use_pava', 'autocorr_noestimate'
            ],
            argstr='--noest',
        ),
        brightness_threshold=dict(argstr='--epith=%d', ),
        design_file=dict(argstr='--pd=%s', ),
        environ=dict(usedefault=True, ),
        fit_armodel=dict(
            xor=[
                'autocorr_estimate_only', 'fit_armodel', 'tukey_window',
                'multitaper_product', 'use_pava', 'autocorr_noestimate'
            ],
            argstr='--ar',
        ),
        full_data=dict(argstr='-v', ),
        in_file=dict(
            mandatory=True,
            argstr='--in=%s',
        ),
        mask_size=dict(argstr='--ms=%d', ),
        multitaper_product=dict(
            xor=[
                'autocorr_estimate_only', 'fit_armodel', 'tukey_window',
                'multitaper_product', 'use_pava', 'autocorr_noestimate'
            ],
            argstr='--mt=%d',
        ),
        output_pwdata=dict(argstr='--outputPWdata', ),
        output_type=dict(),
        results_dir=dict(
            argstr='--rn=%s',
            usedefault=True,
        ),
        smooth_autocorr=dict(argstr='--sa', ),
        threshold=dict(
            usedefault=True,
            argstr='--thr=%f',
        ),
        tukey_window=dict(
            xor=[
                'autocorr_estimate_only', 'fit_armodel', 'tukey_window',
                'multitaper_product', 'use_pava', 'autocorr_noestimate'
            ],
            argstr='--tukey=%d',
        ),
        use_pava=dict(argstr='--pava', ),
    )
    instance = FILMGLS()
    if isinstance(instance.inputs, FILMGLSInputSpec):
        for key, metadata in list(input_map.items()):
            for metakey, value in list(metadata.items()):
                assert getattr(instance.inputs.traits()[key], metakey) == value
    else:
        for key, metadata in list(input_map2.items()):
            for metakey, value in list(metadata.items()):
                assert getattr(instance.inputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
"""The bru2nii module provides basic functions for dicom conversion
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os
from .base import (CommandLine, CommandLineInputSpec, traits, TraitedSpec,
                   isdefined, File, Directory)


class Bru2InputSpec(CommandLineInputSpec):
    input_dir = Directory(
        desc="Input Directory",
        exists=True,
        mandatory=True,
        position=-1,
        argstr="%s")
    actual_size = traits.Bool(
        argstr='-a',
        desc="Keep actual size - otherwise x10 scale so animals match human.")
    force_conversion = traits.Bool(
        argstr='-f',
        desc="Force conversion of localizers images (multiple slice "
        "orientations).")
    compress = traits.Bool(
        argstr='-z', desc='gz compress images (".nii.gz").')
    append_protocol_name = traits.Bool(
        argstr='-p', desc="Append protocol name to output filename.")
    output_filename = traits.Str(
        argstr="-o %s",
        desc='Output filename (".nii" will be appended, or ".nii.gz" if the "-z" compress option is selected)',
        genfile=True)


class Bru2OutputSpec(TraitedSpec):
    nii_file = File(exists=True)


class Bru2(CommandLine):
    """Uses bru2nii's Bru2 to convert Bruker files

    Examples
    ========

    >>> from nipype.interfaces.bru2nii import Bru2
    >>> converter = Bru2()
    >>> converter.inputs.input_dir = "brukerdir"
    >>> converter.cmdline  # doctest: +ELLIPSIS
    'Bru2 -o .../data/brukerdir brukerdir'
    """
    input_spec = Bru2InputSpec
    output_spec = Bru2OutputSpec
    _cmd = "Bru2"

    def _list_outputs(self):
        outputs = self._outputs().get()
        if isdefined(self.inputs.output_filename):
            output_filename1 = os.path.abspath(self.inputs.output_filename)
        else:
            output_filename1 = self._gen_filename('output_filename')
        if self.inputs.compress:
            outputs["nii_file"] = output_filename1 + ".nii.gz"
        else:
            outputs["nii_file"] = output_filename1 + ".nii"
        return outputs

    def _gen_filename(self, name):
        if name == 'output_filename':
            outfile = os.path.join(
                os.getcwd(),
                os.path.basename(os.path.normpath(self.inputs.input_dir)))
            return outfile



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .diffusion import *
from .segmentation import OtsuThresholdSegmentation
from .filtering import OtsuThresholdImageFilter, ResampleScalarVolume
from .converters import BSplineToDeformationField
from .registration import (BSplineDeformableRegistration, AffineRegistration,
                           MultiResolutionAffineRegistration,
                           RigidRegistration, LinearRegistration,
                           ExpertAutomatedRegistration)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..maths import BinaryMaths


def test_BinaryMaths_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=2,
        ),
        internal_datatype=dict(
            argstr='-dt %s',
            position=1,
        ),
        nan2zeros=dict(
            argstr='-nan',
            position=3,
        ),
        operand_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=5,
            xor=['operand_value'],
        ),
        operand_value=dict(
            argstr='%.8f',
            mandatory=True,
            position=5,
            xor=['operand_file'],
        ),
        operation=dict(
            argstr='-%s',
            mandatory=True,
            position=4,
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            hash_files=False,
            position=-2,
        ),
        output_datatype=dict(
            argstr='-odt %s',
            position=-1,
        ),
        output_type=dict(),
    )
    inputs = BinaryMaths.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BinaryMaths_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = BinaryMaths.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Parallel workflow execution via multiprocessing

Support for child processes running as non-daemons based on
http://stackoverflow.com/a/8963618/1183453
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

# Import packages
import os
import multiprocessing as mp
from concurrent.futures import ProcessPoolExecutor
from traceback import format_exception
import sys
from logging import INFO
import gc

from copy import deepcopy
import numpy as np
from ... import logging
from ...utils.profiler import get_system_total_memory_gb
from ..engine import MapNode
from .base import DistributedPluginBase

try:
    from textwrap import indent
except ImportError:

    def indent(text, prefix):
        """ A textwrap.indent replacement for Python < 3.3 """
        if not prefix:
            return text
        splittext = text.splitlines(True)
        return prefix + prefix.join(splittext)


# Init logger
logger = logging.getLogger('nipype.workflow')


# Run node
def run_node(node, updatehash, taskid):
    """Function to execute node.run(), catch and log any errors and
    return the result dictionary

    Parameters
    ----------
    node : nipype Node instance
        the node to run
    updatehash : boolean
        flag for updating hash
    taskid : int
        an identifier for this task

    Returns
    -------
    result : dictionary
        dictionary containing the node runtime results and stats
    """

    # Init variables
    result = dict(result=None, traceback=None, taskid=taskid)

    # Try and execute the node via node.run()
    try:
        result['result'] = node.run(updatehash=updatehash)
    except:  # noqa: E722, intendedly catch all here
        result['traceback'] = format_exception(*sys.exc_info())
        result['result'] = node.result

    # Return the result dictionary
    return result


class MultiProcPlugin(DistributedPluginBase):
    """
    Execute workflow with multiprocessing, not sending more jobs at once
    than the system can support.

    The plugin_args input to run can be used to control the multiprocessing
    execution and defining the maximum amount of memory and threads that
    should be used. When those parameters are not specified,
    the number of threads and memory of the system is used.

    System consuming nodes should be tagged::

      memory_consuming_node.mem_gb = 8
      thread_consuming_node.n_procs = 16

    The default number of threads and memory are set at node
    creation, and are 1 and 0.25GB respectively.

    Currently supported options are:

    - non_daemon: boolean flag to execute as non-daemon processes
    - n_procs: maximum number of threads to be executed in parallel
    - memory_gb: maximum memory (in GB) that can be used at once.
    - raise_insufficient: raise error if the requested resources for
        a node over the maximum `n_procs` and/or `memory_gb`
        (default is ``True``).
    - scheduler: sort jobs topologically (``'tsort'``, default value)
        or prioritize jobs by, first, memory consumption and, second,
        number of threads (``'mem_thread'`` option).
    - mp_context: name of multiprocessing context to use

    """

    def __init__(self, plugin_args=None):
        # Init variables and instance attributes
        super(MultiProcPlugin, self).__init__(plugin_args=plugin_args)
        self._taskresult = {}
        self._task_obj = {}
        self._taskid = 0

        # Cache current working directory and make sure we
        # change to it when workers are set up
        self._cwd = os.getcwd()

        # Read in options or set defaults.
        self.processors = self.plugin_args.get('n_procs', mp.cpu_count())
        self.memory_gb = self.plugin_args.get(
            'memory_gb',  # Allocate 90% of system memory
            get_system_total_memory_gb() * 0.9)
        self.raise_insufficient = self.plugin_args.get('raise_insufficient',
                                                       True)

        # Instantiate different thread pools for non-daemon processes
        logger.debug('[MultiProc] Starting (n_procs=%d, '
                     'mem_gb=%0.2f, cwd=%s)',
                     self.processors, self.memory_gb, self._cwd)

        try:
            mp_context = mp.context.get_context(
                self.plugin_args.get('mp_context'))
            self.pool = ProcessPoolExecutor(max_workers=self.processors,
                                            initializer=os.chdir,
                                            initargs=(self._cwd,),
                                            mp_context=mp_context)
        except (AttributeError, TypeError):
            # Python < 3.7 does not support initialization or contexts
            self.pool = ProcessPoolExecutor(max_workers=self.processors)

        self._stats = None

    def _async_callback(self, args):
        result = args.result()
        self._taskresult[result['taskid']] = result

    def _get_result(self, taskid):
        return self._taskresult.get(taskid)

    def _clear_task(self, taskid):
        del self._task_obj[taskid]

    def _submit_job(self, node, updatehash=False):
        self._taskid += 1

        # Don't allow streaming outputs
        if getattr(node.interface, 'terminal_output', '') == 'stream':
            node.interface.terminal_output = 'allatonce'

        result_future = self.pool.submit(run_node, node, updatehash, self._taskid)
        result_future.add_done_callback(self._async_callback)
        self._task_obj[self._taskid] = result_future

        logger.debug('[MultiProc] Submitted task %s (taskid=%d).',
                     node.fullname, self._taskid)
        return self._taskid

    def _prerun_check(self, graph):
        """Check if any node exeeds the available resources"""
        tasks_mem_gb = []
        tasks_num_th = []
        for node in graph.nodes():
            tasks_mem_gb.append(node.mem_gb)
            tasks_num_th.append(node.n_procs)

        if np.any(np.array(tasks_mem_gb) > self.memory_gb):
            logger.warning(
                'Some nodes exceed the total amount of memory available '
                '(%0.2fGB).', self.memory_gb)
            if self.raise_insufficient:
                raise RuntimeError('Insufficient resources available for job')

        if np.any(np.array(tasks_num_th) > self.processors):
            logger.warning(
                'Some nodes demand for more threads than available (%d).',
                self.processors)
            if self.raise_insufficient:
                raise RuntimeError('Insufficient resources available for job')

    def _postrun_check(self):
        self.pool.shutdown()

    def _check_resources(self, running_tasks):
        """
        Make sure there are resources available
        """
        free_memory_gb = self.memory_gb
        free_processors = self.processors
        for _, jobid in running_tasks:
            free_memory_gb -= min(self.procs[jobid].mem_gb, free_memory_gb)
            free_processors -= min(self.procs[jobid].n_procs, free_processors)

        return free_memory_gb, free_processors

    def _send_procs_to_workers(self, updatehash=False, graph=None):
        """
        Sends jobs to workers when system resources are available.
        """

        # Check to see if a job is available (jobs with all dependencies run)
        # See https://github.com/nipy/nipype/pull/2200#discussion_r141605722
        # See also https://github.com/nipy/nipype/issues/2372
        jobids = np.flatnonzero(~self.proc_done &
                                (self.depidx.sum(axis=0) == 0).__array__())

        # Check available resources by summing all threads and memory used
        free_memory_gb, free_processors = self._check_resources(
            self.pending_tasks)

        stats = (len(self.pending_tasks), len(jobids), free_memory_gb,
                 self.memory_gb, free_processors, self.processors)
        if self._stats != stats:
            tasks_list_msg = ''

            if logger.level <= INFO:
                running_tasks = [
                    '  * %s' % self.procs[jobid].fullname
                    for _, jobid in self.pending_tasks
                ]
                if running_tasks:
                    tasks_list_msg = '\nCurrently running:\n'
                    tasks_list_msg += '\n'.join(running_tasks)
                    tasks_list_msg = indent(tasks_list_msg, ' ' * 21)
            logger.info(
                '[MultiProc] Running %d tasks, and %d jobs ready. Free '
                'memory (GB): %0.2f/%0.2f, Free processors: %d/%d.%s',
                len(self.pending_tasks), len(jobids), free_memory_gb,
                self.memory_gb, free_processors, self.processors,
                tasks_list_msg)
            self._stats = stats

        if free_memory_gb < 0.01 or free_processors == 0:
            logger.debug('No resources available')
            return

        if len(jobids) + len(self.pending_tasks) == 0:
            logger.debug('No tasks are being run, and no jobs can '
                         'be submitted to the queue. Potential deadlock')
            return

        jobids = self._sort_jobs(
            jobids, scheduler=self.plugin_args.get('scheduler'))

        # Run garbage collector before potentially submitting jobs
        gc.collect()

        # Submit jobs
        for jobid in jobids:
            # First expand mapnodes
            if isinstance(self.procs[jobid], MapNode):
                try:
                    num_subnodes = self.procs[jobid].num_subnodes()
                except Exception:
                    traceback = format_exception(*sys.exc_info())
                    self._clean_queue(
                        jobid,
                        graph,
                        result={
                            'result': None,
                            'traceback': traceback
                        })
                    self.proc_pending[jobid] = False
                    continue
                if num_subnodes > 1:
                    submit = self._submit_mapnode(jobid)
                    if not submit:
                        continue

            # Check requirements of this job
            next_job_gb = min(self.procs[jobid].mem_gb, self.memory_gb)
            next_job_th = min(self.procs[jobid].n_procs, self.processors)

            # If node does not fit, skip at this moment
            if next_job_th > free_processors or next_job_gb > free_memory_gb:
                logger.debug('Cannot allocate job %d (%0.2fGB, %d threads).',
                             jobid, next_job_gb, next_job_th)
                continue

            free_memory_gb -= next_job_gb
            free_processors -= next_job_th
            logger.debug('Allocating %s ID=%d (%0.2fGB, %d threads). Free: '
                         '%0.2fGB, %d threads.', self.procs[jobid].fullname,
                         jobid, next_job_gb, next_job_th, free_memory_gb,
                         free_processors)

            # change job status in appropriate queues
            self.proc_done[jobid] = True
            self.proc_pending[jobid] = True

            # If cached and up-to-date just retrieve it, don't run
            if self._local_hash_check(jobid, graph):
                continue

            # updatehash and run_without_submitting are also run locally
            if updatehash or self.procs[jobid].run_without_submitting:
                logger.debug('Running node %s on master thread',
                             self.procs[jobid])
                try:
                    self.procs[jobid].run(updatehash=updatehash)
                except Exception:
                    traceback = format_exception(*sys.exc_info())
                    self._clean_queue(
                        jobid,
                        graph,
                        result={
                            'result': None,
                            'traceback': traceback
                        })

                # Release resources
                self._task_finished_cb(jobid)
                self._remove_node_dirs()
                free_memory_gb += next_job_gb
                free_processors += next_job_th
                # Display stats next loop
                self._stats = None

                # Clean up any debris from running node in main process
                gc.collect()
                continue

            # Task should be submitted to workers
            # Send job to task manager and add to pending tasks
            if self._status_callback:
                self._status_callback(self.procs[jobid], 'start')
            tid = self._submit_job(
                deepcopy(self.procs[jobid]), updatehash=updatehash)
            if tid is None:
                self.proc_done[jobid] = False
                self.proc_pending[jobid] = False
            else:
                self.pending_tasks.insert(0, (tid, jobid))
            # Display stats next loop
            self._stats = None

    def _sort_jobs(self, jobids, scheduler='tsort'):
        if scheduler == 'mem_thread':
            return sorted(
                jobids,
                key=lambda item: (self.procs[item].mem_gb, self.procs[item].n_procs)
            )
        return jobids
